@@ -19,23 +19,6 @@
 
 package org.apache.cayenne.modeler;
 
-import java.awt.event.MouseAdapter;
-import java.awt.event.MouseEvent;
-import java.util.ArrayList;
-import java.util.Comparator;
-import java.util.List;
-
-import javax.swing.Action;
-import javax.swing.JMenuItem;
-import javax.swing.JPopupMenu;
-import javax.swing.JTree;
-import javax.swing.event.TreeSelectionEvent;
-import javax.swing.event.TreeSelectionListener;
-import javax.swing.tree.DefaultMutableTreeNode;
-import javax.swing.tree.MutableTreeNode;
-import javax.swing.tree.TreePath;
-import javax.swing.tree.TreeSelectionModel;
-
 import org.apache.cayenne.configuration.ConfigurationNode;
 import org.apache.cayenne.configuration.DataChannelDescriptor;
 import org.apache.cayenne.configuration.DataNodeDescriptor;
@@ -69,6 +52,8 @@ import org.apache.cayenne.modeler.action.CreateObjEntityAction;
 import org.apache.cayenne.modeler.action.CreateProcedureAction;
 import org.apache.cayenne.modeler.action.CreateQueryAction;
 import org.apache.cayenne.modeler.action.CutAction;
+import org.apache.cayenne.modeler.action.DbEntitySyncAction;
+import org.apache.cayenne.modeler.action.LinkDataMapsAction;
 import org.apache.cayenne.modeler.action.ObjEntitySyncAction;
 import org.apache.cayenne.modeler.action.PasteAction;
 import org.apache.cayenne.modeler.action.RemoveAction;
@@ -93,11 +78,38 @@ import org.apache.cayenne.modeler.util.CayenneAction;
 import org.apache.cayenne.modeler.util.CellRenderers;
 import org.apache.cayenne.modeler.util.Comparators;
 import org.apache.cayenne.project.Project;
-import org.apache.cayenne.query.Query;
+import org.apache.cayenne.map.QueryDescriptor;
 import org.apache.cayenne.reflect.PropertyUtils;
 import org.apache.cayenne.resource.Resource;
-import org.apache.commons.logging.Log;
-import org.apache.commons.logging.LogFactory;
+import org.apache.cayenne.swing.components.TopBorder;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import javax.swing.Action;
+import javax.swing.JMenuItem;
+import javax.swing.JPopupMenu;
+import javax.swing.JTree;
+import javax.swing.SwingUtilities;
+import javax.swing.UIManager;
+import javax.swing.MenuElement;
+import javax.swing.event.TreeExpansionEvent;
+import javax.swing.event.TreeSelectionEvent;
+import javax.swing.event.TreeSelectionListener;
+import javax.swing.event.TreeWillExpandListener;
+import javax.swing.tree.DefaultMutableTreeNode;
+import javax.swing.tree.ExpandVetoException;
+import javax.swing.tree.MutableTreeNode;
+import javax.swing.tree.TreePath;
+import javax.swing.tree.TreeSelectionModel;
+import java.awt.Color;
+import java.awt.Graphics;
+import java.awt.Rectangle;
+import java.awt.dnd.DnDConstants;
+import java.awt.event.MouseAdapter;
+import java.awt.event.MouseEvent;
+import java.util.ArrayList;
+import java.util.Comparator;
+import java.util.List;
 
 /**
  * Panel displaying Cayenne project as a tree.
@@ -109,11 +121,15 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
         ProcedureDisplayListener, MultipleObjectsDisplayListener,
         EmbeddableDisplayListener, EmbeddableListener {
 
-    private static final Log logObj = LogFactory.getLog(ProjectTreeView.class);
+    private static final Logger logObj = LoggerFactory.getLogger(ProjectTreeView.class);
+
+    private static final Color SELECTION_COLOR = UIManager.getColor("Tree.selectionBackground");
 
     protected ProjectController mediator;
     protected TreeSelectionListener treeSelectionListener;
+    protected TreeWillExpandListener treeWillExpandListener;
     protected JPopupMenu popup;
+    private TreeDragSource tds;
 
     public ProjectTreeView(ProjectController mediator) {
         super();
@@ -122,10 +138,13 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
         initView();
         initController();
         initFromModel(Application.getInstance().getProject());
+        this.tds = new TreeDragSource(this, DnDConstants.ACTION_COPY, mediator);
     }
 
     private void initView() {
         setCellRenderer(CellRenderers.treeRenderer());
+        setOpaque(false);
+        setBorder(TopBorder.create());
     }
 
     private void initController() {
@@ -136,17 +155,28 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
 
                 if (paths != null) {
                     if (paths.length > 1) {
+                        ConfigurationNode projectParentPath = null;
                         ConfigurationNode[] projectPaths = new ConfigurationNode[paths.length];
+                        boolean commonParentPath = true;
+
                         for (int i = 0; i < paths.length; i++) {
                             projectPaths[i] = createProjectPath(paths[i]);
+
+                            TreePath parentPath = paths[i].getParentPath();
+                            if(i>0 && parentPath != null && !parentPath.equals(paths[i - 1].getParentPath())) {
+                                commonParentPath = false;
+                            }
                         }
 
-                        mediator
-                                .fireMultipleObjectsDisplayEvent(new MultipleObjectsDisplayEvent(
+                        if(commonParentPath) {
+                            TreePath parentPath = paths[0].getParentPath();
+                            projectParentPath = createProjectPath(parentPath);
+                        }
+
+                        mediator.fireMultipleObjectsDisplayEvent(new MultipleObjectsDisplayEvent(
                                         this,
-                                        projectPaths));
-                    }
-                    else if (paths.length == 1) {
+                                        projectPaths, projectParentPath));
+                    } else if (paths.length == 1) {
                         processSelection(paths[0]);
                     }
                 }
@@ -164,9 +194,28 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
             }
         };
 
-        addTreeSelectionListener(treeSelectionListener);
+        treeWillExpandListener = new TreeWillExpandListener() {
+            @Override
+            public void treeWillExpand(TreeExpansionEvent e) throws ExpandVetoException {
+                TreePath path = e.getPath();
+                if (!isPathSelected(path) && !isSelectionEmpty()) {
+                    setSelectionPath(path);
+                }
+            }
 
-        addMouseListener(new PopupHandler());
+            @Override
+            public void treeWillCollapse(TreeExpansionEvent e) throws ExpandVetoException {
+                TreePath path = e.getPath();
+                if (!isPathSelected(path) && !isSelectionEmpty()) {
+                    setSelectionPath(path);
+                }
+            }
+        };
+
+        addTreeSelectionListener(treeSelectionListener);
+        addTreeWillExpandListener(treeWillExpandListener);
+
+        addMouseListener(new MouseClickHandler());
 
         mediator.addDomainListener(this);
         mediator.addDomainDisplayListener(this);
@@ -184,6 +233,7 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
         mediator.addProcedureDisplayListener(this);
         mediator.addQueryListener(this);
         mediator.addQueryDisplayListener(this);
+        mediator.addMultipleObjectsDisplayListener(this);
 
         mediator.getApplication().getActionManager().setupCutCopyPaste(
                 this,
@@ -204,7 +254,7 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
     /**
      * Returns tree model cast to ProjectTreeModel.
      */
-    ProjectTreeModel getProjectModel() {
+    public ProjectTreeModel getProjectModel() {
         return (ProjectTreeModel) getModel();
     }
 
@@ -232,7 +282,6 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
 
         // Project - return the name of top file
         if (value instanceof Project) {
-
             Resource resource = ((Project) value).getConfigurationResource();
             return (resource != null) ? resource.getURL().getPath() : "";
         }
@@ -240,20 +289,12 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
         // read name property
         try {
             if (value instanceof Embeddable) {
-                return (value != null) ? String.valueOf(PropertyUtils.getProperty(
-                        value,
-                        "className")) : "";
+                return String.valueOf(PropertyUtils.getProperty(value, "className"));
             }
 
-            return (value != null) ? String.valueOf(PropertyUtils.getProperty(
-                    value,
-                    "name")) : "";
-
-        }
-        catch (Exception e) {
-            String objectClass = (value == null) ? "(unknown)" : value
-                    .getClass()
-                    .getName();
+            return (value != null) ? String.valueOf(PropertyUtils.getProperty(value, "name")) : "";
+        } catch (Exception e) {
+            String objectClass = (value == null) ? "(unknown)" : value.getClass().getName();
             logObj.warn("Exception reading property 'name', class " + objectClass, e);
             return "";
         }
@@ -329,16 +370,42 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
         });
     }
 
-    public void currentObjectsChanged(
-            MultipleObjectsDisplayEvent e,
-            Application application) {
+    public void currentObjectsChanged(MultipleObjectsDisplayEvent e, Application application) {
+        if (e.getSource() == this || e.getParentNode() == null) {
+            return;
+        }
+
+        ConfigurationNode[] nodes = e.getNodes();
+        TreePath[] treePaths = new TreePath[nodes.length];
+
+        for (int i = 0; i < nodes.length; i++) {
+            DefaultMutableTreeNode treeNode = getProjectModel().getNodeForObjectPath(new Object[] {e.getParentNode(), nodes[i]});
+            if (treeNode != null) {
+                treePaths[i] = new TreePath(treeNode.getPath());
+            } else if (e.getParentNode() == nodes[i]) {
+                treeNode = getProjectModel().getNodeForObjectPath(new Object[] {e.getParentNode()});
+                treePaths[i] = new TreePath(treeNode.getPath());
+            }
+        }
+
+        if (!isVisible(treePaths[0])) {
+            makeVisible(treePaths[0]);
+
+            Rectangle bounds = getPathBounds(treePaths[0]);
+            if (bounds != null) {
+                bounds.height = getVisibleRect().height;
+                scrollRectToVisible(bounds);
+            }
+        }
+
+        setSelectionPaths(treePaths);
     }
 
     public void procedureAdded(ProcedureEvent e) {
 
         DefaultMutableTreeNode node = getProjectModel().getNodeForObjectPath(
                 new Object[] {
-                        (DataChannelDescriptor) mediator.getProject().getRootNode(),
+                        mediator.getProject().getRootNode(),
                         e.getProcedure().getDataMap()
                 });
 
@@ -388,7 +455,7 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
             return;
         }
 
-        Query query = e.getQuery();
+        QueryDescriptor query = e.getQuery();
         DefaultMutableTreeNode currentNode = new DefaultMutableTreeNode(query, false);
         positionNode(node, currentNode, Comparators.getDataMapChildrenComparator());
         showNode(currentNode);
@@ -401,7 +468,7 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
                     e.getDomain() != null
                             ? e.getDomain()
                             : (DataChannelDescriptor) mediator.getProject().getRootNode(),
-                    e.getQuery()
+                    e.getQuery().getDataMap(), e.getQuery()
             };
 
             updateNode(path);
@@ -447,63 +514,55 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
         if (node != null) {
 
             if (e.isNameChange()) {
-                positionNode((DefaultMutableTreeNode) node.getParent(), node, Comparators
-                        .getDataDomainChildrenComparator());
+                positionNode((DefaultMutableTreeNode) node.getParent(), node,
+                        Comparators.getDataDomainChildrenComparator());
                 showNode(node);
-            }
-            else {
+            } else {
 
                 getProjectModel().nodeChanged(node);
 
-                DataChannelDescriptor domain = (DataChannelDescriptor) mediator
-                        .getProject()
-                        .getRootNode();
+                DataChannelDescriptor domain = (DataChannelDescriptor) mediator.getProject().getRootNode();
 
                 // check for DataMap additions/removals...
-                Object[] mapsName = e.getDataNode().getDataMapNames().toArray();
+                String[] mapsName = e.getDataNode().getDataMapNames().toArray(new String[0]);
                 int mapCount = mapsName.length;
 
                 // DataMap was linked
                 if (mapCount > node.getChildCount()) {
 
-                    for (int i = 0; i < mapCount; i++) {
+                    for (String aMapsName : mapsName) {
                         boolean found = false;
                         for (int j = 0; j < node.getChildCount(); j++) {
-                            DefaultMutableTreeNode child = (DefaultMutableTreeNode) node
-                                    .getChildAt(j);
-                            if (domain.getDataMap(mapsName[i].toString()) == child
-                                    .getUserObject()) {
+                            DefaultMutableTreeNode child = (DefaultMutableTreeNode) node.getChildAt(j);
+                            if (domain.getDataMap(aMapsName) == child.getUserObject()) {
                                 found = true;
                                 break;
                             }
                         }
 
                         if (!found) {
-                            DefaultMutableTreeNode newMapNode = new DefaultMutableTreeNode(
-                                    domain.getDataMap(mapsName[i].toString()),
-                                    false);
-                            positionNode(node, newMapNode, Comparators
-                                    .getNamedObjectComparator());
+                            DefaultMutableTreeNode newMapNode =
+                                    new DefaultMutableTreeNode(domain.getDataMap(aMapsName), false);
+                            positionNode(node, newMapNode, Comparators.getNamedObjectComparator());
                             break;
                         }
                     }
-                }
-                // DataMap was unlinked
-                else if (mapCount < node.getChildCount()) {
-                    for (int j = 0; j < node.getChildCount(); j++) {
+                } else if (mapCount < node.getChildCount()) {
+                    // DataMap was unlinked
+                    int j = 0;
+                    while (j < node.getChildCount()) {
                         boolean found = false;
                         DefaultMutableTreeNode child;
                         child = (DefaultMutableTreeNode) node.getChildAt(j);
                         Object obj = child.getUserObject();
-                        for (int i = 0; i < mapCount; i++) {
-                            if (domain.getDataMap(mapsName[i].toString()) == obj) {
+                        for (Object aMapsName : mapsName) {
+                            if (domain.getDataMap(aMapsName.toString()) == obj) {
                                 found = true;
-                                break;
+                                j++;
                             }
                         }
                         if (!found) {
                             removeNode(child);
-                            break;
                         }
                     }
                 }
@@ -593,8 +652,7 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
         mediator.getEntityResolver().removeDataMap(e.getDataMap());
 
         // Clean up map from the nodes
-        for (DataNodeDescriptor dataNode : new ArrayList<DataNodeDescriptor>(
-                dataChannelDescriptor.getNodeDescriptors())) {
+        for (DataNodeDescriptor dataNode : new ArrayList<>(dataChannelDescriptor.getNodeDescriptors())) {
             removeNode(new Object[] {
                     dataChannelDescriptor, dataNode, map
             });
@@ -738,7 +796,17 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
     /** Makes node current, visible and selected. */
     protected void showNode(DefaultMutableTreeNode node) {
         TreePath path = new TreePath(node.getPath());
-        scrollPathToVisible(path);
+
+        if (!isVisible(path)) {
+            makeVisible(path);
+
+            Rectangle bounds = getPathBounds(path);
+            if (bounds != null) {
+                bounds.height = getVisibleRect().height;
+                scrollRectToVisible(bounds);
+            }
+        }
+
         setSelectionPath(path);
     }
 
@@ -788,8 +856,7 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
             return;
         }
 
-        DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) path
-                .getLastPathComponent();
+        DefaultMutableTreeNode currentNode = (DefaultMutableTreeNode) path.getLastPathComponent();
 
         Object[] data = getUserObjects(currentNode);
         if (data.length == 0) {
@@ -807,70 +874,61 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
             mediator.fireDomainDisplayEvent(new DomainDisplayEvent(
                     this,
                     (DataChannelDescriptor) obj));
-        }
-        else if (obj instanceof DataMap) {
+        } else if (obj instanceof DataMap) {
             if (data.length == 2) {
                 mediator.fireDataMapDisplayEvent(new DataMapDisplayEvent(
                         this,
                         (DataMap) obj,
                         (DataChannelDescriptor) mediator.getProject().getRootNode(),
                         (DataNodeDescriptor) data[data.length - 2]));
-            }
-            else if (data.length == 1) {
+            } else if (data.length == 1) {
                 mediator.fireDataMapDisplayEvent(new DataMapDisplayEvent(
                         this,
                         (DataMap) obj,
                         (DataChannelDescriptor) mediator.getProject().getRootNode()));
             }
-        }
-        else if (obj instanceof DataNodeDescriptor) {
+        } else if (obj instanceof DataNodeDescriptor) {
             if (data.length == 1) {
                 mediator.fireDataNodeDisplayEvent(new DataNodeDisplayEvent(
                         this,
                         (DataChannelDescriptor) mediator.getProject().getRootNode(),
                         (DataNodeDescriptor) obj));
             }
-        }
-        else if (obj instanceof Entity) {
+        } else if (obj instanceof Entity) {
             EntityDisplayEvent e = new EntityDisplayEvent(this, (Entity) obj);
             e.setUnselectAttributes(true);
             if (data.length == 3) {
                 e.setDataMap((DataMap) data[data.length - 2]);
                 e.setDomain((DataChannelDescriptor) mediator.getProject().getRootNode());
                 e.setDataNode((DataNodeDescriptor) data[data.length - 3]);
-            }
-            else if (data.length == 2) {
+            } else if (data.length == 2) {
                 e.setDataMap((DataMap) data[data.length - 2]);
                 e.setDomain((DataChannelDescriptor) mediator.getProject().getRootNode());
             }
 
             if (obj instanceof ObjEntity) {
                 mediator.fireObjEntityDisplayEvent(e);
-            }
-            else if (obj instanceof DbEntity) {
+            } else if (obj instanceof DbEntity) {
                 mediator.fireDbEntityDisplayEvent(e);
             }
-        }
-        else if (obj instanceof Embeddable) {
+        } else if (obj instanceof Embeddable) {
             EmbeddableDisplayEvent e = new EmbeddableDisplayEvent(
                     this,
                     (Embeddable) obj,
                     (DataMap) data[data.length - 2],
                     (DataChannelDescriptor) mediator.getProject().getRootNode());
             mediator.fireEmbeddableDisplayEvent(e);
-        }
-        else if (obj instanceof Procedure) {
+        } else if (obj instanceof Procedure) {
             ProcedureDisplayEvent e = new ProcedureDisplayEvent(
                     this,
                     (Procedure) obj,
                     (DataMap) data[data.length - 2],
                     (DataChannelDescriptor) mediator.getProject().getRootNode());
             mediator.fireProcedureDisplayEvent(e);
-        }
-        else if (obj instanceof Query) {
+        } else if (obj instanceof QueryDescriptor) {
             QueryDisplayEvent e = new QueryDisplayEvent(
                     this,
-                    (Query) obj,
+                    (QueryDescriptor) obj,
                     (DataMap) data[data.length - 2],
                     (DataChannelDescriptor) mediator.getProject().getRootNode());
             mediator.fireQueryDisplayEvent(e);
@@ -884,7 +942,7 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
      * root. That is the array of actual objects rather than wrappers.
      */
     private Object[] getUserObjects(DefaultMutableTreeNode node) {
-        List list = new ArrayList();
+        List<Object> list = new ArrayList<>();
         while (!node.isRoot()) {
             list.add(0, node.getUserObject());
             node = (DefaultMutableTreeNode) node.getParent();
@@ -913,8 +971,7 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
         removeTreeSelectionListener(treeSelectionListener);
         try {
             getProjectModel().positionNode(parent, treeNode, comparator);
-        }
-        finally {
+        } finally {
             addTreeSelectionListener(treeSelectionListener);
         }
     }
@@ -940,9 +997,9 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
         popup.add(buildMenu(CreateQueryAction.class));
         popup.addSeparator();
         popup.add(buildMenu(ObjEntitySyncAction.class));
-        popup.addSeparator();
+        popup.add(buildMenu(DbEntitySyncAction.class));
+        popup.add(buildMenu(LinkDataMapsAction.class));
         popup.add(buildMenu(RemoveAction.class));
-        popup.addSeparator();
         popup.add(buildMenu(CutAction.class));
         popup.add(buildMenu(CopyAction.class));
         popup.add(buildMenu(PasteAction.class));
@@ -953,7 +1010,7 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
     /**
      * Creates and returns an menu item associated with the key.
      * 
-     * @param key action key
+     * @param actionType action type
      */
     private JMenuItem buildMenu(Class<? extends Action> actionType) {
         CayenneAction action = (CayenneAction) mediator
@@ -964,28 +1021,40 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
     }
 
     /**
-     * Class to handle right-click and show popup for selected tree row
+     * Class to handle:
+     *  - right-click and show popup for selected tree row
+     *  - left click row selection based on full row length (instead of default selection based on label size)
      */
-    class PopupHandler extends MouseAdapter {
+    class MouseClickHandler extends MouseAdapter {
+
+        void selectRowForEvent(MouseEvent e) {
+            int closestRow = getClosestRowForLocation(e.getX(), e.getY());
+            Rectangle closestRowBounds = getRowBounds(closestRow);
+            if(e.getY() >= closestRowBounds.getY()
+                    && e.getY() < closestRowBounds.getY() + closestRowBounds.getHeight()
+                    && !isRowSelected(closestRow)) {
+                setSelectionRow(closestRow);
+            }
+        }
 
         @Override
         public void mousePressed(MouseEvent e) {
+            if (SwingUtilities.isLeftMouseButton(e)) {
+                selectRowForEvent(e);
+            }
+
             mouseReleased(e);
         }
 
         @Override
         public void mouseReleased(MouseEvent e) {
             if (e.isPopupTrigger()) {
-                if (popup == null)
+                // Selecting specified row
+                selectRowForEvent(e);
+                if (popup == null) {
                     popup = createJPopupMenu();
-
-                /**
-                 * Selecting specified row
-                 */
-                int row = getRowForLocation(e.getX(), e.getY());
-                if (row != -1 && !isRowSelected(row))
-                    setSelectionRow(row);
-
+                }
+                popupMenuFilter();
                 popup.show(ProjectTreeView.this, e.getX(), e.getY());
             }
         }
@@ -1052,4 +1121,38 @@ public class ProjectTreeView extends JTree implements DomainDisplayListener,
         });
 
     }
+
+    // Filter all disabled actions in popupMenu, but skip Cut-Copy-Paste block. It should always exist.
+    public void popupMenuFilter() {
+        Action cutAction = mediator.getApplication().getActionManager().getAction(CutAction.class);
+        for (MenuElement element : popup.getSubElements()) {
+            JMenuItem item = (JMenuItem) element;
+            if (!item.getAction().equals(cutAction)) {
+                item.setVisible(item.isEnabled());
+            } else {
+                break;
+            }
+        }
+    }
+
+    public TreeDragSource getTds() {
+        return tds;
+    }
+
+    @Override
+    public void paintComponent(Graphics g) {
+        g.setColor(Color.white);
+        g.fillRect(0, 0, getWidth(), getHeight());
+        if (getSelectionCount() > 0) {
+            g.setColor(SELECTION_COLOR);
+            int[] rows = getSelectionRows();
+            if(rows != null) {
+                for (int i : rows) {
+                    Rectangle r = getRowBounds(i);
+                    g.fillRect(0, r.y, getWidth(), r.height);
+                }
+            }
+        }
+        super.paintComponent(g);
+    }
 }
