@@ -18,23 +18,10 @@
  ****************************************************************/
 package org.apache.cayenne.modeler.action;
 
-import java.awt.Toolkit;
-import java.awt.datatransfer.FlavorEvent;
-import java.awt.datatransfer.FlavorListener;
-import java.awt.datatransfer.UnsupportedFlavorException;
-import java.awt.event.ActionEvent;
-import java.awt.event.KeyEvent;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
-
-import javax.swing.JOptionPane;
-import javax.swing.KeyStroke;
-import javax.swing.undo.UndoableEdit;
-
 import org.apache.cayenne.configuration.ConfigurationNode;
 import org.apache.cayenne.configuration.DataChannelDescriptor;
 import org.apache.cayenne.configuration.DataNodeDescriptor;
+import org.apache.cayenne.dbsync.naming.NameBuilder;
 import org.apache.cayenne.map.DataMap;
 import org.apache.cayenne.map.DbAttribute;
 import org.apache.cayenne.map.DbEntity;
@@ -48,28 +35,38 @@ import org.apache.cayenne.map.ObjEntity;
 import org.apache.cayenne.map.ObjRelationship;
 import org.apache.cayenne.map.Procedure;
 import org.apache.cayenne.map.ProcedureParameter;
+import org.apache.cayenne.map.QueryDescriptor;
+import org.apache.cayenne.map.event.MapEvent;
 import org.apache.cayenne.modeler.Application;
 import org.apache.cayenne.modeler.ProjectController;
 import org.apache.cayenne.modeler.dialog.ErrorDebugDialog;
 import org.apache.cayenne.modeler.dialog.query.QueryType;
+import org.apache.cayenne.modeler.editor.ObjCallbackMethod;
+import org.apache.cayenne.modeler.event.CallbackMethodEvent;
 import org.apache.cayenne.modeler.undo.PasteCompoundUndoableEdit;
 import org.apache.cayenne.modeler.undo.PasteUndoableEdit;
 import org.apache.cayenne.modeler.util.CayenneAction;
 import org.apache.cayenne.modeler.util.CayenneTransferable;
-import org.apache.cayenne.query.AbstractQuery;
-import org.apache.cayenne.query.EJBQLQuery;
 import org.apache.cayenne.query.Query;
 
+import javax.swing.KeyStroke;
+import javax.swing.undo.UndoableEdit;
+import java.awt.Toolkit;
+import java.awt.datatransfer.FlavorEvent;
+import java.awt.datatransfer.FlavorListener;
+import java.awt.datatransfer.UnsupportedFlavorException;
+import java.awt.event.ActionEvent;
+import java.awt.event.KeyEvent;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+
 /**
  * Action for pasting entities, queries etc. from the system buffer
  */
 public class PasteAction extends CayenneAction implements FlavorListener {
 
-    private static final String COPY_PREFIX = "Copy of ";
-
-    public static String getActionName() {
-        return "Paste";
-    }
+    private static final String COPY_PATTERN = "copy of %s (%d)";
 
     /**
      * Constructor for PasteAction
@@ -82,9 +79,13 @@ public class PasteAction extends CayenneAction implements FlavorListener {
         Toolkit.getDefaultToolkit().getSystemClipboard().addFlavorListener(this);
     }
 
+    public static String getActionName() {
+        return "Paste";
+    }
+
     @Override
     public String getIconName() {
-        return "icon-paste.gif";
+        return "icon-paste.png";
     }
 
     @Override
@@ -105,6 +106,10 @@ public class PasteAction extends CayenneAction implements FlavorListener {
 
             Object currentObject = getProjectController().getCurrentObject();
 
+            if(content instanceof DataMap) {
+                currentObject = getProjectController().getProject().getRootNode();
+            }
+
             if (content != null && currentObject != null) {
                 DataChannelDescriptor domain = (DataChannelDescriptor) getProjectController()
                         .getProject()
@@ -123,19 +128,16 @@ public class PasteAction extends CayenneAction implements FlavorListener {
                                 currentObject,
                                 o));
                     }
-                }
-                else {
+                } else {
                     paste(currentObject, content);
                     undoableEdit = new PasteUndoableEdit(domain, map, currentObject, content);
                 }
 
                 application.getUndoManager().addEdit(undoableEdit);
             }
-        }
-        catch (UnsupportedFlavorException ufe) {
+        } catch (UnsupportedFlavorException ufe) {
             // do nothing
-        }
-        catch (Exception ex) {
+        } catch (Exception ex) {
             ErrorDebugDialog.guiException(ex);
         }
     }
@@ -152,7 +154,7 @@ public class PasteAction extends CayenneAction implements FlavorListener {
     public void paste(
             Object where,
             Object content,
-            DataChannelDescriptor domain,
+            DataChannelDescriptor dataChannelDescriptor,
             DataMap map) {
         final ProjectController mediator = getProjectController();
 
@@ -169,9 +171,11 @@ public class PasteAction extends CayenneAction implements FlavorListener {
             // paste DataMap to DataDomain or DataNode
             DataMap dataMap = ((DataMap) content);
 
-            dataMap
-                    .setName(getFreeName(new DataMapNameChecker(domain), dataMap
-                            .getName()));
+            dataMap.setName(NameBuilder
+                    .builder(dataMap, dataChannelDescriptor)
+                    .baseName(dataMap.getName())
+                    .dupesPattern(COPY_PATTERN)
+                    .name());
 
             /**
              * Update all names in the new DataMap, so that they would not conflict with
@@ -180,24 +184,30 @@ public class PasteAction extends CayenneAction implements FlavorListener {
 
             // add some intelligence - if we rename an entity, we should rename all links
             // to it as well
-            Map<String, String> renamedDbEntities = new HashMap<String, String>();
-            Map<String, String> renamedObjEntities = new HashMap<String, String>();
-
-            Map<String, String> renamedEmbeddables = new HashMap<String, String>();
+            Map<String, String> renamedDbEntities = new HashMap<>();
+            Map<String, String> renamedObjEntities = new HashMap<>();
+            Map<String, String> renamedEmbeddables = new HashMap<>();
 
             for (DbEntity dbEntity : dataMap.getDbEntities()) {
                 String oldName = dbEntity.getName();
-                dbEntity.setName(getFreeName(new DbEntityNameChecker(domain), dbEntity
-                        .getName()));
+                dbEntity.setName(NameBuilder
+                        .builder(dbEntity, dataMap)
+                        .baseName(dbEntity.getName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
 
                 if (!oldName.equals(dbEntity.getName())) {
                     renamedDbEntities.put(oldName, dbEntity.getName());
                 }
             }
+
             for (ObjEntity objEntity : dataMap.getObjEntities()) {
                 String oldName = objEntity.getName();
-                objEntity.setName(getFreeName(new ObjEntityNameChecker(domain), objEntity
-                        .getName()));
+                objEntity.setName(NameBuilder
+                        .builder(objEntity, dataMap)
+                        .baseName(objEntity.getName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
 
                 if (!oldName.equals(objEntity.getName())) {
                     renamedObjEntities.put(oldName, objEntity.getName());
@@ -206,9 +216,11 @@ public class PasteAction extends CayenneAction implements FlavorListener {
 
             for (Embeddable embeddable : dataMap.getEmbeddables()) {
                 String oldName = embeddable.getClassName();
-                embeddable.setClassName(getFreeName(
-                        new EmbeddableNameChecker(domain),
-                        embeddable.getClassName()));
+                embeddable.setClassName(NameBuilder
+                        .builder(embeddable, dataMap)
+                        .baseName(embeddable.getClassName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
 
                 if (!oldName.equals(embeddable.getClassName())) {
                     renamedEmbeddables.put(oldName, embeddable.getClassName());
@@ -216,13 +228,18 @@ public class PasteAction extends CayenneAction implements FlavorListener {
             }
 
             for (Procedure procedure : dataMap.getProcedures()) {
-                procedure.setName(getFreeName(new ProcedureNameChecker(domain), procedure
-                        .getName()));
+                procedure.setName(NameBuilder
+                        .builder(procedure, dataMap)
+                        .baseName(procedure.getName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
             }
-            for (Query query : dataMap.getQueries()) {
-                ((AbstractQuery) query).setName(getFreeName(
-                        new QueryNameChecker(domain),
-                        query.getName()));
+
+            for (QueryDescriptor query : dataMap.getQueryDescriptors()) {
+                query.setName(NameBuilder.builder(query, dataMap)
+                        .baseName(query.getName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
             }
 
             // if an entity was renamed, we rename all links to it too
@@ -254,8 +271,7 @@ public class PasteAction extends CayenneAction implements FlavorListener {
             }
 
             mediator.addDataMap(this, dataMap);
-        }
-        else if (where instanceof DataMap) {
+        } else if (where instanceof DataMap) {
             // paste DbEntity to DataMap
             final DataMap dataMap = ((DataMap) where);
 
@@ -264,17 +280,21 @@ public class PasteAction extends CayenneAction implements FlavorListener {
 
             if (content instanceof DbEntity) {
                 DbEntity dbEntity = (DbEntity) content;
-                dbEntity.setName(getFreeName(new DbEntityNameChecker(domain), dbEntity
-                        .getName()));
+                dbEntity.setName(NameBuilder
+                        .builder(dbEntity, dataMap)
+                        .baseName(dbEntity.getName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
 
                 dataMap.addDbEntity(dbEntity);
                 CreateDbEntityAction.fireDbEntityEvent(this, mediator, dbEntity);
-            }
-            else if (content instanceof ObjEntity) {
+            } else if (content instanceof ObjEntity) {
                 // paste ObjEntity to DataMap
                 ObjEntity objEntity = (ObjEntity) content;
-                objEntity.setName(getFreeName(new ObjEntityNameChecker(domain), objEntity
-                        .getName()));
+                objEntity.setName(NameBuilder.builder(objEntity, dataMap)
+                        .baseName(objEntity.getName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
 
                 dataMap.addObjEntity(objEntity);
                 CreateObjEntityAction.fireObjEntityEvent(
@@ -282,13 +302,14 @@ public class PasteAction extends CayenneAction implements FlavorListener {
                         mediator,
                         dataMap,
                         objEntity);
-            }
-            else if (content instanceof Embeddable) {
+            } else if (content instanceof Embeddable) {
                 // paste Embeddable to DataMap
                 Embeddable embeddable = (Embeddable) content;
-                embeddable.setClassName(getFreeName(
-                        new EmbeddableNameChecker(domain),
-                        embeddable.getClassName()));
+                embeddable.setClassName(NameBuilder
+                        .builder(embeddable, dataMap)
+                        .baseName(embeddable.getClassName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
 
                 dataMap.addEmbeddable(embeddable);
                 CreateEmbeddableAction.fireEmbeddableEvent(
@@ -296,31 +317,26 @@ public class PasteAction extends CayenneAction implements FlavorListener {
                         mediator,
                         dataMap,
                         embeddable);
-            }
-            else if (content instanceof EJBQLQuery) {
-                EJBQLQuery query = (EJBQLQuery) content;
+            } else if (content instanceof QueryDescriptor) {
+                QueryDescriptor query = (QueryDescriptor) content;
 
-                query.setName(getFreeName(new QueryNameChecker(domain), query.getName()));
+                query.setName(NameBuilder
+                        .builder(query, dataMap)
+                        .dupesPattern(COPY_PATTERN)
+                        .baseName(query.getName())
+                        .name());
                 query.setDataMap(dataMap);
 
-                dataMap.addQuery(query);
+                dataMap.addQueryDescriptor(query);
                 QueryType.fireQueryEvent(this, mediator, dataMap, query);
-            }
-            else if (content instanceof Query) {
-                // paste Query to DataMap
-                AbstractQuery query = (AbstractQuery) content;
-
-                query.setName(getFreeName(new QueryNameChecker(domain), query.getName()));
-                query.setDataMap(dataMap);
-
-                dataMap.addQuery(query);
-                QueryType.fireQueryEvent(this, mediator, dataMap, query);
-            }
-            else if (content instanceof Procedure) {
+            } else if (content instanceof Procedure) {
                 // paste Procedure to DataMap
                 Procedure procedure = (Procedure) content;
-                procedure.setName(getFreeName(new ProcedureNameChecker(domain), procedure
-                        .getName()));
+                procedure.setName(NameBuilder
+                        .builder(procedure, dataMap)
+                        .dupesPattern(COPY_PATTERN)
+                        .baseName(procedure.getName())
+                        .name());
 
                 dataMap.addProcedure(procedure);
                 CreateProcedureAction.fireProcedureEvent(
@@ -329,30 +345,27 @@ public class PasteAction extends CayenneAction implements FlavorListener {
                         dataMap,
                         procedure);
             }
-        }
-        else if (where instanceof DbEntity) {
+        } else if (where instanceof DbEntity) {
             final DbEntity dbEntity = (DbEntity) where;
 
-            // attrs and rels must be unique in entity namespace
-            FreeNameChecker checker = new FreeNameChecker() {
-
-                public boolean isNameFree(String name) {
-                    return dbEntity.getAttribute(name) == null
-                            && dbEntity.getRelationship(name) == null;
-                }
-            };
-
             if (content instanceof DbAttribute) {
                 DbAttribute attr = (DbAttribute) content;
-                attr.setName(getFreeName(checker, attr.getName()));
+                attr.setName(NameBuilder
+                        .builder(attr, dbEntity)
+                        .dupesPattern(COPY_PATTERN)
+                        .baseName(attr.getName())
+                        .name());
 
                 dbEntity.addAttribute(attr);
                 CreateAttributeAction.fireDbAttributeEvent(this, mediator, mediator
                         .getCurrentDataMap(), dbEntity, attr);
-            }
-            else if (content instanceof DbRelationship) {
+            } else if (content instanceof DbRelationship) {
                 DbRelationship rel = (DbRelationship) content;
-                rel.setName(getFreeName(checker, rel.getName()));
+                rel.setName(NameBuilder
+                        .builder(rel, dbEntity)
+                        .baseName(rel.getName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
 
                 dbEntity.addRelationship(rel);
                 CreateRelationshipAction.fireDbRelationshipEvent(
@@ -361,30 +374,27 @@ public class PasteAction extends CayenneAction implements FlavorListener {
                         dbEntity,
                         rel);
             }
-        }
-        else if (where instanceof ObjEntity) {
-            final ObjEntity objEntity = (ObjEntity) where;
-
-            // attrs and rels must be unique in entity namespace
-            FreeNameChecker checker = new FreeNameChecker() {
-
-                public boolean isNameFree(String name) {
-                    return objEntity.getAttribute(name) == null
-                            && objEntity.getRelationship(name) == null;
-                }
-            };
+        } else if (where instanceof ObjEntity) {
+            ObjEntity objEntity = (ObjEntity) where;
 
             if (content instanceof ObjAttribute) {
                 ObjAttribute attr = (ObjAttribute) content;
-                attr.setName(getFreeName(checker, attr.getName()));
+                attr.setName(NameBuilder
+                        .builder(attr, objEntity)
+                        .baseName(attr.getName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
 
                 objEntity.addAttribute(attr);
                 CreateAttributeAction.fireObjAttributeEvent(this, mediator, mediator
                         .getCurrentDataMap(), objEntity, attr);
-            }
-            else if (content instanceof ObjRelationship) {
+            } else if (content instanceof ObjRelationship) {
                 ObjRelationship rel = (ObjRelationship) content;
-                rel.setName(getFreeName(checker, rel.getName()));
+                rel.setName(NameBuilder
+                        .builder(rel, objEntity)
+                        .baseName(rel.getName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
 
                 objEntity.addRelationship(rel);
                 CreateRelationshipAction.fireObjRelationshipEvent(
@@ -392,23 +402,37 @@ public class PasteAction extends CayenneAction implements FlavorListener {
                         mediator,
                         objEntity,
                         rel);
+            } else if (content instanceof ObjCallbackMethod) {
+                ObjCallbackMethod method = (ObjCallbackMethod) content;
+
+                method.setName(NameBuilder
+                        .builderForCallbackMethod(objEntity)
+                        .baseName(method.getName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
+
+                objEntity.getCallbackMap()
+                        .getCallbackDescriptor(mediator.getCurrentCallbackType().getType())
+                        .addCallbackMethod(method.getName());
+
+                CallbackMethodEvent ce = new CallbackMethodEvent(
+                        this,
+                        null,
+                        method.getName(),
+                        MapEvent.ADD);
+
+                getProjectController().fireCallbackMethodEvent(ce);
             }
-        }
-
-        else if (where instanceof Embeddable) {
+        } else if (where instanceof Embeddable) {
             final Embeddable embeddable = (Embeddable) where;
 
-            // attrs and rels must be unique in entity namespace
-            FreeNameChecker checker = new FreeNameChecker() {
-
-                public boolean isNameFree(String name) {
-                    return embeddable.getAttribute(name) == null;
-                }
-            };
-
             if (content instanceof EmbeddableAttribute) {
                 EmbeddableAttribute attr = (EmbeddableAttribute) content;
-                attr.setName(getFreeName(checker, attr.getName()));
+                attr.setName(NameBuilder
+                        .builder(attr, embeddable)
+                        .baseName(attr.getName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
 
                 embeddable.addAttribute(attr);
                 CreateAttributeAction.fireEmbeddableAttributeEvent(
@@ -418,28 +442,18 @@ public class PasteAction extends CayenneAction implements FlavorListener {
                         attr);
             }
 
-        }
-
-        else if (where instanceof Procedure) {
+        } else if (where instanceof Procedure) {
             // paste param to procedure
             final Procedure procedure = (Procedure) where;
 
             if (content instanceof ProcedureParameter) {
                 ProcedureParameter param = (ProcedureParameter) content;
 
-                param.setName(getFreeName(new FreeNameChecker() {
-
-                    public boolean isNameFree(String name) {
-                        for (ProcedureParameter existingParam : procedure
-                                .getCallParameters()) {
-                            if (name.equals(existingParam.getName())) {
-                                return false;
-                            }
-                        }
-
-                        return true;
-                    }
-                }, param.getName()));
+                param.setName(NameBuilder
+                        .builder(param, procedure)
+                        .baseName(param.getName())
+                        .dupesPattern(COPY_PATTERN)
+                        .name());
 
                 procedure.addCallParameter(param);
                 CreateProcedureParameterAction.fireProcedureParameterEvent(
@@ -455,24 +469,10 @@ public class PasteAction extends CayenneAction implements FlavorListener {
     private void clearDataMapCache(DataMap dataMap) {
         MappingNamespace ns = dataMap.getNamespace();
         if (ns instanceof EntityResolver) {
-            ((EntityResolver) ns).clearCache();
+            ((EntityResolver) ns).refreshMappingCache();
         }
     }
 
-    /**
-     * Finds avaliable name for an object
-     */
-    private String getFreeName(FreeNameChecker checker, String defName) {
-        String name = defName;
-
-        for (int i = 0; !checker.isNameFree(name); name = COPY_PREFIX
-                + defName
-                + (i == 0 ? "" : " (" + i + ")"), i++)
-            ;
-
-        return name;
-    }
-
     /**
      * Returns <code>true</code> if last object in the path contains a removable object.
      */
@@ -511,7 +511,7 @@ public class PasteAction extends CayenneAction implements FlavorListener {
             }
 
             /**
-             * Checking all avaliable pairs source-pasting object
+             * Checking all available pairs source-pasting object
              */
 
             return ((currentObject instanceof DataChannelDescriptor || currentObject instanceof DataNodeDescriptor) && content instanceof DataMap)
@@ -520,12 +520,15 @@ public class PasteAction extends CayenneAction implements FlavorListener {
                     (currentObject instanceof DataMap && isTreeLeaf(content))
                     ||
 
+                    (currentObject instanceof DataMap && content instanceof DataMap)
+                    ||
+
                     (currentObject instanceof DbEntity && (content instanceof DbAttribute
                             || content instanceof DbRelationship || isTreeLeaf(content)))
                     ||
 
                     (currentObject instanceof ObjEntity && (content instanceof ObjAttribute
-                            || content instanceof ObjRelationship || isTreeLeaf(content)))
+                            || content instanceof ObjRelationship || content instanceof ObjCallbackMethod || isTreeLeaf(content)))
                     ||
 
                     (currentObject instanceof Embeddable && (content instanceof EmbeddableAttribute || isTreeLeaf(content)))
@@ -534,9 +537,8 @@ public class PasteAction extends CayenneAction implements FlavorListener {
                     (currentObject instanceof Procedure
                             && (content instanceof ProcedureParameter || isTreeLeaf(content)) ||
 
-                    (currentObject instanceof Query && isTreeLeaf(content)));
-        }
-        catch (Exception ex) {
+                            (currentObject instanceof Query && isTreeLeaf(content)));
+        } catch (Exception ex) {
             return false;
         }
     }
@@ -555,156 +557,4 @@ public class PasteAction extends CayenneAction implements FlavorListener {
     public void flavorsChanged(FlavorEvent e) {
         updateState();
     }
-
-    /**
-     * Interface for checking that specified name is free in superior DataMap, Entity etc.
-     * and therefore can be used for new object
-     */
-    interface FreeNameChecker {
-
-        boolean isNameFree(String name);
-    }
-
-    /**
-     * FreeNameChecker implementation for choosing DataMap names
-     */
-    class DataMapNameChecker implements FreeNameChecker {
-
-        DataChannelDescriptor domain;
-
-        public DataMapNameChecker(DataChannelDescriptor domain) {
-            this.domain = domain;
-        }
-
-        public boolean isNameFree(String name) {
-            return domain.getDataMap(name) == null;
-        }
-    }
-
-    /**
-     * FreeNameChecker implementation for choosing DbEntity names
-     */
-    class DbEntityNameChecker implements FreeNameChecker {
-
-        DataChannelDescriptor domain;
-
-        public DbEntityNameChecker(DataChannelDescriptor domain) {
-            this.domain = domain;
-        }
-
-        public boolean isNameFree(String name) {
-            /**
-             * Name mast be unique through all DataDomain, for EntityResolver to work
-             * correctlys
-             */
-            for (DataMap map : domain.getDataMaps()) {
-                if (map.getDbEntity(name) != null) {
-                    return false;
-                }
-            }
-
-            return true;
-        }
-    }
-
-    /**
-     * FreeNameChecker implementation for choosing ObjEntity names
-     */
-    class ObjEntityNameChecker implements FreeNameChecker {
-
-        DataChannelDescriptor domain;
-
-        public ObjEntityNameChecker(DataChannelDescriptor domain) {
-            this.domain = domain;
-        }
-
-        public boolean isNameFree(String name) {
-            /**
-             * Name mast be unique through all DataDomain, for EntityResolver to work
-             * correctly
-             */
-            for (DataMap map : domain.getDataMaps()) {
-                if (map.getObjEntity(name) != null) {
-                    return false;
-                }
-            }
-
-            return true;
-        }
-    }
-
-    class EmbeddableNameChecker implements FreeNameChecker {
-
-        DataChannelDescriptor domain;
-
-        public EmbeddableNameChecker(DataChannelDescriptor domain) {
-            this.domain = domain;
-        }
-
-        public boolean isNameFree(String name) {
-            /**
-             * Name mast be unique through all DataDomain, for EntityResolver to work
-             * correctly
-             */
-            for (DataMap map : domain.getDataMaps()) {
-                if (map.getEmbeddable(name) != null) {
-                    return false;
-                }
-            }
-
-            return true;
-        }
-    }
-
-    /**
-     * FreeNameChecker implementation for choosing Procedure names
-     */
-    class ProcedureNameChecker implements FreeNameChecker {
-
-        DataChannelDescriptor domain;
-
-        public ProcedureNameChecker(DataChannelDescriptor domain) {
-            this.domain = domain;
-        }
-
-        public boolean isNameFree(String name) {
-            /**
-             * Name mast be unique through all DataDomain, for EntityResolver to work
-             * correctly
-             */
-            for (DataMap map : domain.getDataMaps()) {
-                if (map.getNamespace().getProcedure(name) != null) {
-                    return false;
-                }
-            }
-
-            return true;
-        }
-    }
-
-    /**
-     * FreeNameChecker implementation for choosing Query names
-     */
-    class QueryNameChecker implements FreeNameChecker {
-
-        DataChannelDescriptor domain;
-
-        public QueryNameChecker(DataChannelDescriptor domain) {
-            this.domain = domain;
-        }
-
-        public boolean isNameFree(String name) {
-            /**
-             * Name mast be unique through all DataDomain, for EntityResolver to work
-             * correctly
-             */
-            for (DataMap map : domain.getDataMaps()) {
-                if (map.getNamespace().getQuery(name) != null) {
-                    return false;
-                }
-            }
-
-            return true;
-        }
-    }
 }
