@@ -19,20 +19,10 @@
 
 package org.apache.cayenne.modeler.dialog.codegen;
 
-import java.io.File;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.Iterator;
-import java.util.prefs.Preferences;
-
-import javax.swing.JButton;
-import javax.swing.JFileChooser;
-import javax.swing.JOptionPane;
-import javax.swing.JTextField;
-
-import org.apache.cayenne.gen.ClassGenerationAction;
+import org.apache.cayenne.CayenneRuntimeException;
 import org.apache.cayenne.gen.ArtifactsGenerationMode;
-import org.apache.cayenne.map.Attribute;
+import org.apache.cayenne.gen.ClassGenerationAction;
+import org.apache.cayenne.map.DataMap;
 import org.apache.cayenne.map.Embeddable;
 import org.apache.cayenne.map.EmbeddableAttribute;
 import org.apache.cayenne.map.EmbeddedAttribute;
@@ -53,14 +43,27 @@ import org.apache.cayenne.validation.ValidationFailure;
 import org.apache.cayenne.validation.ValidationResult;
 import org.apache.commons.collections.Predicate;
 
+import javax.swing.JButton;
+import javax.swing.JFileChooser;
+import javax.swing.JOptionPane;
+import javax.swing.JTextField;
+import java.io.File;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.LinkedList;
+import java.util.Map;
+import java.util.Set;
+import java.util.prefs.Preferences;
+
 /**
  * A mode-specific part of the code generation dialog.
  * 
  */
 public abstract class GeneratorController extends CayenneController {
 
-    protected DataMapDefaults preferences;
-    protected String mode = ArtifactsGenerationMode.ENTITY.getLabel();
+    protected String mode = ArtifactsGenerationMode.ALL.getLabel();
+    protected Map<DataMap, DataMapDefaults> mapPreferences;
+    private String outputPath;
 
     public GeneratorController(CodeGeneratorControllerBase parent) {
         super(parent);
@@ -71,21 +74,42 @@ public abstract class GeneratorController extends CayenneController {
     }
 
     public String getOutputPath() {
-        return preferences.getOutputPath();
+        return outputPath;
     }
 
     public void setOutputPath(String path) {
-        preferences.setOutputPath(path);
+        String old = this.outputPath;
+        this.outputPath = path;
+        if (this.outputPath != null && !this.outputPath.equals(old)) {
+            updatePreferences(path);
+        }
+    }
+
+    public void updatePreferences(String path) {
+        if (mapPreferences == null)
+            return;
+        Set<DataMap> keys = mapPreferences.keySet();
+        for (DataMap key : keys) {
+            mapPreferences
+                    .get(key)
+                    .setOutputPath(path);
+        }
+    }
+
+    public void setMapPreferences(Map<DataMap, DataMapDefaults> mapPreferences) {
+        this.mapPreferences = mapPreferences;
+    }
+
+    public Map<DataMap, DataMapDefaults> getMapPreferences() {
+        return this.mapPreferences;
     }
 
     protected void initBindings(BindingBuilder bindingBuilder) {
 
         initOutputFolder();
 
-        JTextField outputFolder = ((GeneratorControllerPanel) getView())
-                .getOutputFolder();
-        JButton outputSelect = ((GeneratorControllerPanel) getView())
-                .getSelectOutputFolder();
+        JTextField outputFolder = ((GeneratorControllerPanel) getView()).getOutputFolder();
+        JButton outputSelect = ((GeneratorControllerPanel) getView()).getSelectOutputFolder();
 
         outputFolder.setText(getOutputPath());
         bindingBuilder.bindToAction(outputSelect, "selectOutputFolderAction()");
@@ -98,88 +122,95 @@ public abstract class GeneratorController extends CayenneController {
 
     protected abstract GeneratorControllerPanel createView();
 
-    protected abstract DataMapDefaults createDefaults();
+    protected abstract void createDefaults();
 
     /**
-     * Creates an appropriate subclass of {@link ClassGenerationAction}, returning it in
-     * an unconfigured state. Configuration is performed by {@link #createGenerator()}
-     * method.
+     * Creates an appropriate subclass of {@link ClassGenerationAction},
+     * returning it in an unconfigured state. Configuration is performed by
+     * {@link #createGenerator()} method.
      */
     protected abstract ClassGenerationAction newGenerator();
 
     /**
      * Creates a class generator for provided selections.
      */
-    public ClassGenerationAction createGenerator() {
+    public Collection<ClassGenerationAction> createGenerator() {
 
         File outputDir = getOutputDir();
 
         // no destination folder
         if (outputDir == null) {
-            JOptionPane.showMessageDialog(
-                    this.getView(),
-                    "Select directory for source files.");
+            JOptionPane.showMessageDialog(this.getView(), "Select directory for source files.");
             return null;
         }
 
         // no such folder
         if (!outputDir.exists() && !outputDir.mkdirs()) {
-            JOptionPane.showMessageDialog(this.getView(), "Can't create directory "
-                    + outputDir
+            JOptionPane.showMessageDialog(this.getView(), "Can't create directory " + outputDir
                     + ". Select a different one.");
             return null;
         }
 
         // not a directory
         if (!outputDir.isDirectory()) {
-            JOptionPane.showMessageDialog(this.getView(), outputDir
-                    + " is not a valid directory.");
+            JOptionPane.showMessageDialog(this.getView(), outputDir + " is not a valid directory.");
             return null;
         }
 
         // remove generic entities...
-        Collection<ObjEntity> entities = new ArrayList<ObjEntity>(getParentController()
-                .getSelectedEntities());
-        Iterator<ObjEntity> it = entities.iterator();
-        while (it.hasNext()) {
-            if (it.next().isGeneric()) {
-                it.remove();
+        Collection<ObjEntity> selectedEntities = new ArrayList<>(getParentController().getSelectedEntities());
+        for(ObjEntity objEntity : selectedEntities) {
+            if(objEntity.isGeneric()) {
+                selectedEntities.remove(objEntity);
             }
         }
 
-        ClassGenerationAction generator = newGenerator();
-        generator.setArtifactsGenerationMode(mode);
-        generator.setDataMap(getParentController().getDataMap());
-        generator.addEntities(entities);
-        generator.addEmbeddables(getParentController().getSelectedEmbeddables());
-        generator.addQueries(getParentController().getDataMap().getQueries());
+        Collection<ClassGenerationAction> generators = new ArrayList<>();
+        Collection<StandardPanelComponent> dataMapLines = ((GeneratorControllerPanel) getView()).getDataMapLines();
+        for (DataMap map : getParentController().getDataMaps()) {
+            try {
+                ClassGenerationAction generator = newGenerator();
 
-        Preferences preferences = application.getPreferencesNode(
-                GeneralPreferences.class,
-                "");
+                if(getParentController().getSelectedDataMaps().contains(map)) {
+                    mode = ArtifactsGenerationMode.ALL.getLabel();
+                } else {
+                    mode = ArtifactsGenerationMode.ENTITY.getLabel();
+                }
 
-        if (preferences != null) {
-            generator.setEncoding(preferences.get(
-                    GeneralPreferences.ENCODING_PREFERENCE,
-                    null));
+                generator.setArtifactsGenerationMode(mode);
+                generator.setDataMap(map);
+
+                LinkedList<ObjEntity> objEntities = new LinkedList<>(map.getObjEntities());
+                objEntities.retainAll(selectedEntities);
+                generator.addEntities(objEntities);
+
+                LinkedList<Embeddable> embeddables = new LinkedList<>(map.getEmbeddables());
+                embeddables.retainAll(getParentController().getSelectedEmbeddables());
+                generator.addEmbeddables(embeddables);
+
+                generator.addQueries(map.getQueryDescriptors());
+
+                Preferences preferences = application.getPreferencesNode(GeneralPreferences.class, "");
+
+                if (preferences != null) {
+                    generator.setEncoding(preferences.get(GeneralPreferences.ENCODING_PREFERENCE, null));
+
+                }
+
+                generator.setDestDir(outputDir);
+                generator.setMakePairs(true);
+
+                generators.add(generator);
+            } catch (CayenneRuntimeException exception) {
+                JOptionPane.showMessageDialog(this.getView(), exception.getUnlabeledMessage());
+                return null;
+            }
         }
 
-        generator.setDestDir(outputDir);
-        generator.setMakePairs(true);
-
-        String superPackage = ((GeneratorControllerPanel) getView())
-                .getSuperclassPackage()
-                .getText();
-        if (!Util.isEmptyString(superPackage)) {
-            generator.setSuperPkg(superPackage);
-        }
-
-        return generator;
+        return generators;
     }
 
-    public void validateEmbeddable(
-            ValidationResult validationBuffer,
-            Embeddable embeddable) {
+    public void validateEmbeddable(ValidationResult validationBuffer, Embeddable embeddable) {
         ValidationFailure embeddableFailure = validateEmbeddable(embeddable);
         if (embeddableFailure != null) {
             validationBuffer.addFailure(embeddableFailure);
@@ -198,33 +229,25 @@ public abstract class GeneratorController extends CayenneController {
     private ValidationFailure validateEmbeddableAttribute(EmbeddableAttribute attribute) {
         String name = attribute.getEmbeddable().getClassName();
 
-        ValidationFailure emptyName = BeanValidationFailure.validateNotEmpty(
-                name,
-                "attribute.name",
+        ValidationFailure emptyName = BeanValidationFailure.validateNotEmpty(name, "attribute.name",
                 attribute.getName());
         if (emptyName != null) {
             return emptyName;
         }
 
-        ValidationFailure badName = CodeValidationUtil.validateJavaIdentifier(
-                name,
-                "attribute.name",
+        ValidationFailure badName = CodeValidationUtil.validateJavaIdentifier(name, "attribute.name",
                 attribute.getName());
         if (badName != null) {
             return badName;
         }
 
-        ValidationFailure emptyType = BeanValidationFailure.validateNotEmpty(
-                name,
-                "attribute.type",
+        ValidationFailure emptyType = BeanValidationFailure.validateNotEmpty(name, "attribute.type",
                 attribute.getType());
         if (emptyType != null) {
             return emptyType;
         }
 
-        ValidationFailure badType = BeanValidationFailure.validateJavaClassName(
-                name,
-                "attribute.type",
+        ValidationFailure badType = BeanValidationFailure.validateJavaClassName(name, "attribute.type",
                 attribute.getType());
         if (badType != null) {
             return badType;
@@ -233,21 +256,17 @@ public abstract class GeneratorController extends CayenneController {
         return null;
     }
 
-    protected ValidationFailure validateEmbeddable(Embeddable embeddable) {
+    private ValidationFailure validateEmbeddable(Embeddable embeddable) {
 
         String name = embeddable.getClassName();
 
-        ValidationFailure emptyClass = BeanValidationFailure.validateNotEmpty(
-                name,
-                "className",
+        ValidationFailure emptyClass = BeanValidationFailure.validateNotEmpty(name, "className",
                 embeddable.getClassName());
         if (emptyClass != null) {
             return emptyClass;
         }
 
-        ValidationFailure badClass = BeanValidationFailure.validateJavaClassName(
-                name,
-                "className",
+        ValidationFailure badClass = BeanValidationFailure.validateJavaClassName(name, "className",
                 embeddable.getClassName());
         if (badClass != null) {
             return badClass;
@@ -256,19 +275,15 @@ public abstract class GeneratorController extends CayenneController {
         return null;
     }
 
-    public void validateEntity(
-            ValidationResult validationBuffer,
-            ObjEntity entity,
-            boolean clientValidation) {
+    public void validateEntity(ValidationResult validationBuffer, ObjEntity entity, boolean clientValidation) {
 
-        ValidationFailure entityFailure = validateEntity(clientValidation ? entity
-                .getClientEntity() : entity);
+        ValidationFailure entityFailure = validateEntity(clientValidation ? entity.getClientEntity() : entity);
         if (entityFailure != null) {
             validationBuffer.addFailure(entityFailure);
             return;
         }
 
-        for (Attribute attribute : entity.getAttributes()) {
+        for (ObjAttribute attribute : entity.getAttributes()) {
             if (attribute instanceof EmbeddedAttribute) {
                 EmbeddedAttribute embeddedAttribute = (EmbeddedAttribute) attribute;
                 for (ObjAttribute subAttribute : embeddedAttribute.getAttributes()) {
@@ -278,10 +293,9 @@ public abstract class GeneratorController extends CayenneController {
                         return;
                     }
                 }
-            }
-            else {
+            } else {
 
-                ValidationFailure failure = validateAttribute((ObjAttribute) attribute);
+                ValidationFailure failure = validateAttribute(attribute);
                 if (failure != null) {
                     validationBuffer.addFailure(failure);
                     return;
@@ -298,7 +312,7 @@ public abstract class GeneratorController extends CayenneController {
         }
     }
 
-    protected ValidationFailure validateEntity(ObjEntity entity) {
+    private ValidationFailure validateEntity(ObjEntity entity) {
 
         String name = entity.getName();
 
@@ -306,26 +320,20 @@ public abstract class GeneratorController extends CayenneController {
             return new SimpleValidationFailure(name, "Generic class");
         }
 
-        ValidationFailure emptyClass = BeanValidationFailure.validateNotEmpty(
-                name,
-                "className",
-                entity.getClassName());
+        ValidationFailure emptyClass = BeanValidationFailure.validateNotEmpty(name, "className", entity.getClassName());
         if (emptyClass != null) {
             return emptyClass;
         }
 
-        ValidationFailure badClass = BeanValidationFailure.validateJavaClassName(
-                name,
-                "className",
+        ValidationFailure badClass = BeanValidationFailure.validateJavaClassName(name, "className",
                 entity.getClassName());
         if (badClass != null) {
             return badClass;
         }
 
         if (entity.getSuperClassName() != null) {
-            ValidationFailure badSuperClass = BeanValidationFailure
-                    .validateJavaClassName(name, "superClassName", entity
-                            .getSuperClassName());
+            ValidationFailure badSuperClass = BeanValidationFailure.validateJavaClassName(name, "superClassName",
+                    entity.getSuperClassName());
             if (badSuperClass != null) {
                 return badSuperClass;
             }
@@ -334,37 +342,29 @@ public abstract class GeneratorController extends CayenneController {
         return null;
     }
 
-    protected ValidationFailure validateAttribute(ObjAttribute attribute) {
+    private ValidationFailure validateAttribute(ObjAttribute attribute) {
 
         String name = attribute.getEntity().getName();
 
-        ValidationFailure emptyName = BeanValidationFailure.validateNotEmpty(
-                name,
-                "attribute.name",
+        ValidationFailure emptyName = BeanValidationFailure.validateNotEmpty(name, "attribute.name",
                 attribute.getName());
         if (emptyName != null) {
             return emptyName;
         }
 
-        ValidationFailure badName = CodeValidationUtil.validateJavaIdentifier(
-                name,
-                "attribute.name",
+        ValidationFailure badName = CodeValidationUtil.validateJavaIdentifier(name, "attribute.name",
                 attribute.getName());
         if (badName != null) {
             return badName;
         }
 
-        ValidationFailure emptyType = BeanValidationFailure.validateNotEmpty(
-                name,
-                "attribute.type",
+        ValidationFailure emptyType = BeanValidationFailure.validateNotEmpty(name, "attribute.type",
                 attribute.getType());
         if (emptyType != null) {
             return emptyType;
         }
 
-        ValidationFailure badType = BeanValidationFailure.validateJavaClassName(
-                name,
-                "attribute.type",
+        ValidationFailure badType = BeanValidationFailure.validateJavaClassName(name, "attribute.type",
                 attribute.getType());
         if (badType != null) {
             return badType;
@@ -373,7 +373,7 @@ public abstract class GeneratorController extends CayenneController {
         return null;
     }
 
-    protected ValidationFailure validateEmbeddedAttribute(ObjAttribute attribute) {
+    private ValidationFailure validateEmbeddedAttribute(ObjAttribute attribute) {
 
         String name = attribute.getEntity().getName();
 
@@ -385,49 +385,35 @@ public abstract class GeneratorController extends CayenneController {
         int beginIndex = attributes[0].length();
         String attr = attribute.getName().substring(beginIndex + 1);
 
-        ValidationFailure emptyEmbeddedName = BeanValidationFailure.validateNotEmpty(
-                name,
-                "attribute.name",
+        ValidationFailure emptyEmbeddedName = BeanValidationFailure.validateNotEmpty(name, "attribute.name",
                 nameEmbeddedAttribute);
         if (emptyEmbeddedName != null) {
             return emptyEmbeddedName;
         }
 
-        ValidationFailure badEmbeddedName = CodeValidationUtil.validateJavaIdentifier(
-                name,
-                "attribute.name",
+        ValidationFailure badEmbeddedName = CodeValidationUtil.validateJavaIdentifier(name, "attribute.name",
                 nameEmbeddedAttribute);
         if (badEmbeddedName != null) {
             return badEmbeddedName;
         }
 
-        ValidationFailure emptyName = BeanValidationFailure.validateNotEmpty(
-                name,
-                "attribute.name",
-                attr);
+        ValidationFailure emptyName = BeanValidationFailure.validateNotEmpty(name, "attribute.name", attr);
         if (emptyName != null) {
             return emptyName;
         }
 
-        ValidationFailure badName = CodeValidationUtil.validateJavaIdentifier(
-                name,
-                "attribute.name",
-                attr);
+        ValidationFailure badName = CodeValidationUtil.validateJavaIdentifier(name, "attribute.name", attr);
         if (badName != null) {
             return badName;
         }
 
-        ValidationFailure emptyType = BeanValidationFailure.validateNotEmpty(
-                name,
-                "attribute.type",
+        ValidationFailure emptyType = BeanValidationFailure.validateNotEmpty(name, "attribute.type",
                 attribute.getType());
         if (emptyType != null) {
             return emptyType;
         }
 
-        ValidationFailure badType = BeanValidationFailure.validateJavaClassName(
-                name,
-                "attribute.type",
+        ValidationFailure badType = BeanValidationFailure.validateJavaClassName(name, "attribute.type",
                 attribute.getType());
         if (badType != null) {
             return badType;
@@ -436,23 +422,17 @@ public abstract class GeneratorController extends CayenneController {
         return null;
     }
 
-    protected ValidationFailure validateRelationship(
-            ObjRelationship relationship,
-            boolean clientValidation) {
+    private ValidationFailure validateRelationship(ObjRelationship relationship, boolean clientValidation) {
 
         String name = relationship.getSourceEntity().getName();
 
-        ValidationFailure emptyName = BeanValidationFailure.validateNotEmpty(
-                name,
-                "relationship.name",
+        ValidationFailure emptyName = BeanValidationFailure.validateNotEmpty(name, "relationship.name",
                 relationship.getName());
         if (emptyName != null) {
             return emptyName;
         }
 
-        ValidationFailure badName = CodeValidationUtil.validateJavaIdentifier(
-                name,
-                "relationship.name",
+        ValidationFailure badName = CodeValidationUtil.validateJavaIdentifier(name, "relationship.name",
                 relationship.getName());
         if (badName != null) {
             return badName;
@@ -460,7 +440,7 @@ public abstract class GeneratorController extends CayenneController {
 
         if (!relationship.isToMany()) {
 
-            ObjEntity targetEntity = (ObjEntity) relationship.getTargetEntity();
+            ObjEntity targetEntity = relationship.getTargetEntity();
 
             if (clientValidation && targetEntity != null) {
                 targetEntity = targetEntity.getClientEntity();
@@ -468,24 +448,16 @@ public abstract class GeneratorController extends CayenneController {
 
             if (targetEntity == null) {
 
-                return new BeanValidationFailure(
-                        name,
-                        "relationship.targetEntity",
-                        "No target entity");
-            }
-            else if (!targetEntity.isGeneric()) {
-                ValidationFailure emptyClass = BeanValidationFailure.validateNotEmpty(
-                        name,
-                        "relationship.targetEntity.className",
-                        targetEntity.getClassName());
+                return new BeanValidationFailure(name, "relationship.targetEntity", "No target entity");
+            } else if (!targetEntity.isGeneric()) {
+                ValidationFailure emptyClass = BeanValidationFailure.validateNotEmpty(name,
+                        "relationship.targetEntity.className", targetEntity.getClassName());
                 if (emptyClass != null) {
                     return emptyClass;
                 }
 
-                ValidationFailure badClass = BeanValidationFailure.validateJavaClassName(
-                        name,
-                        "relationship.targetEntity.className",
-                        targetEntity.getClassName());
+                ValidationFailure badClass = BeanValidationFailure.validateJavaClassName(name,
+                        "relationship.targetEntity.className", targetEntity.getClassName());
                 if (badClass != null) {
                     return badClass;
                 }
@@ -499,81 +471,34 @@ public abstract class GeneratorController extends CayenneController {
      * Returns a predicate for default entity selection in a given mode.
      */
     public Predicate getDefaultClassFilter() {
-        final ObjEntity selectedEntity = Application
-                .getInstance()
-                .getFrameController()
-                .getProjectController()
-                .getCurrentObjEntity();
-
-        final Embeddable selectedEmbeddable = Application
-                .getInstance()
-                .getFrameController()
-                .getProjectController()
-                .getCurrentEmbeddable();
-
-        // select a single entity
-        if (selectedEntity != null) {
-            final boolean hasProblem = getParentController().getProblem(
-                    selectedEntity.getName()) != null;
-
-            return new Predicate() {
-
-                public boolean evaluate(Object object) {
-                    return !hasProblem && object == selectedEntity;
+        return new Predicate() {
+            @Override
+            public boolean evaluate(Object o) {
+                if (o instanceof ObjEntity) {
+                    return GeneratorController.this.getParentController().getProblem(((ObjEntity) o).getName()) == null;
                 }
-            };
-        }
-        // select a single embeddable
-        else if (selectedEmbeddable != null) {
-            final boolean hasProblem = getParentController().getProblem(
-                    selectedEmbeddable.getClassName()) != null;
 
-            return new Predicate() {
-
-                public boolean evaluate(Object object) {
-                    return !hasProblem && object == selectedEmbeddable;
+                if (o instanceof Embeddable) {
+                    return GeneratorController.this.getParentController().getProblem(((Embeddable) o).getClassName()) == null;
                 }
-            };
-        }
-        // select all entities
-        else {
 
-            return new Predicate() {
-
-                public boolean evaluate(Object object) {
-                    if (object instanceof ObjEntity) {
-                        return getParentController().getProblem(
-                                ((ObjEntity) object).getName()) == null;
-                    }
-
-                    if (object instanceof Embeddable) {
-                        return getParentController().getProblem(
-                                ((Embeddable) object).getClassName()) == null;
-                    }
-
-                    return false;
-                }
-            };
-        }
+                return false;
+            }
+        };
     }
 
-    public File getOutputDir() {
+    private File getOutputDir() {
         String dir = ((GeneratorControllerPanel) getView()).getOutputFolder().getText();
         return dir != null ? new File(dir) : new File(System.getProperty("user.dir"));
     }
 
-    public DataMapDefaults getPreferences() {
-        return preferences;
-    }
-
     /**
-     * An action method that pops up a file chooser dialog to pick the generation
-     * directory.
+     * An action method that pops up a file chooser dialog to pick the
+     * generation directory.
      */
     public void selectOutputFolderAction() {
 
-        JTextField outputFolder = ((GeneratorControllerPanel) getView())
-                .getOutputFolder();
+        JTextField outputFolder = ((GeneratorControllerPanel) getView()).getOutputFolder();
 
         String currentDir = outputFolder.getText();
 
@@ -584,12 +509,8 @@ public abstract class GeneratorController extends CayenneController {
         // guess start directory
         if (!Util.isEmptyString(currentDir)) {
             chooser.setCurrentDirectory(new File(currentDir));
-        }
-        else {
-            FSPath lastDir = Application
-                    .getInstance()
-                    .getFrameController()
-                    .getLastDirectory();
+        } else {
+            FSPath lastDir = Application.getInstance().getFrameController().getLastDirectory();
             lastDir.updateChooser(chooser);
         }
 
@@ -605,28 +526,20 @@ public abstract class GeneratorController extends CayenneController {
     }
 
     private void initOutputFolder() {
-
-        String path = null;
-        if (preferences.getOutputPath() == null) {
+        String path;
+        if (getOutputPath() == null) {
             if (System.getProperty("cayenne.cgen.destdir") != null) {
                 setOutputPath(System.getProperty("cayenne.cgen.destdir"));
-            }
-            else {
+            } else {
                 // init default directory..
-                FSPath lastPath = Application
-                        .getInstance()
-                        .getFrameController()
-                        .getLastDirectory();
+                FSPath lastPath = Application.getInstance().getFrameController().getLastDirectory();
 
                 path = checkDefaultMavenResourceDir(lastPath, "test");
 
-                if (path != null
-                        || (path = checkDefaultMavenResourceDir(lastPath, "main")) != null) {
+                if (path != null || (path = checkDefaultMavenResourceDir(lastPath, "main")) != null) {
                     setOutputPath(path);
-                }
-                else {
-                    File lastDir = (lastPath != null) ? lastPath
-                            .getExistingDirectory(false) : null;
+                } else {
+                    File lastDir = (lastPath != null) ? lastPath.getExistingDirectory(false) : null;
                     setOutputPath(lastDir != null ? lastDir.getAbsolutePath() : null);
                 }
             }
@@ -640,12 +553,10 @@ public abstract class GeneratorController extends CayenneController {
         if (idx < 0) {
             return null;
         }
-        return path.substring(0, idx)
-                + buildFilePath("src", dirType, "java")
-                + path.substring(idx + resourcePath.length());
+        return path.substring(0, idx) + buildFilePath("src", dirType, "java");
     }
 
-    private static final String buildFilePath(String... pathElements) {
+    private static String buildFilePath(String... pathElements) {
         if (pathElements.length == 0) {
             return "";
         }
@@ -655,4 +566,5 @@ public abstract class GeneratorController extends CayenneController {
         }
         return path.toString();
     }
+
 }
