@@ -19,7 +19,7 @@
 package org.apache.cayenne.modeler.dialog.objentity;
 
 import org.apache.cayenne.CayenneRuntimeException;
-import org.apache.cayenne.map.Attribute;
+import org.apache.cayenne.dbsync.naming.ObjectNameGenerator;
 import org.apache.cayenne.map.DbEntity;
 import org.apache.cayenne.map.DbRelationship;
 import org.apache.cayenne.map.Entity;
@@ -29,15 +29,16 @@ import org.apache.cayenne.map.ObjRelationship;
 import org.apache.cayenne.map.Relationship;
 import org.apache.cayenne.map.event.RelationshipEvent;
 import org.apache.cayenne.modeler.Application;
+import org.apache.cayenne.modeler.ClassLoadingService;
 import org.apache.cayenne.modeler.ProjectController;
 import org.apache.cayenne.modeler.dialog.ResolveDbRelationshipDialog;
 import org.apache.cayenne.modeler.util.CayenneController;
 import org.apache.cayenne.modeler.util.Comparators;
-import org.apache.cayenne.modeler.util.EntityTreeFilter;
 import org.apache.cayenne.modeler.util.EntityTreeModel;
+import org.apache.cayenne.modeler.util.EntityTreeRelationshipFilter;
 import org.apache.cayenne.modeler.util.MultiColumnBrowser;
+import org.apache.cayenne.modeler.util.NameGeneratorPreferences;
 import org.apache.cayenne.util.DeleteRuleUpdater;
-import org.apache.cayenne.util.NamedObjectFactory;
 import org.apache.cayenne.util.Util;
 
 import javax.swing.JOptionPane;
@@ -54,18 +55,17 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
-import java.util.Vector;
+import java.util.Objects;
+
+public class ObjRelationshipInfo extends CayenneController implements TreeSelectionListener {
 
-public class ObjRelationshipInfo extends CayenneController implements
-    TreeSelectionListener {
-    
     static final String COLLECTION_TYPE_MAP = "java.util.Map";
     static final String COLLECTION_TYPE_SET = "java.util.Set";
     static final String COLLECTION_TYPE_COLLECTION = "java.util.Collection";
     static final String DEFAULT_MAP_KEY = "ID (default)";
-    
+
     protected ObjRelationship relationship;
-    
+
     protected List<DbRelationship> dbRelationships;
 
     protected List<DbRelationship> savedDbRelationships;
@@ -78,7 +78,7 @@ public class ObjRelationshipInfo extends CayenneController implements
     protected ObjRelationshipInfoView view;
     protected String currentPath;
     protected ProjectController mediator;
-    
+
     /**
      * Starts options dialog.
      */
@@ -90,11 +90,11 @@ public class ObjRelationshipInfo extends CayenneController implements
         centerView();
         view.setVisible(true);
     }
-   
-    public ObjRelationshipInfo(ProjectController mediator,ObjRelationship relationship) {
+
+    public ObjRelationshipInfo(ProjectController mediator, ObjRelationship relationship) {
         super(mediator);
         this.view = new ObjRelationshipInfoView(mediator);
-        this.mediator=mediator;
+        this.mediator = mediator;
         getPathBrowser().addTreeSelectionListener(this);
         view.sourceEntityLabel.setText(relationship.getSourceEntity().getName());
         this.relationship = relationship;
@@ -105,7 +105,7 @@ public class ObjRelationshipInfo extends CayenneController implements
             targetCollection = ObjRelationship.DEFAULT_COLLECTION_TYPE;
         }
 
-        this.objectTarget = (ObjEntity) relationship.getTargetEntity();
+        this.objectTarget = relationship.getTargetEntity();
         if (objectTarget != null) {
             updateTargetCombo(objectTarget.getDbEntity());
             view.targetCombo.setSelectedItem(objectTarget.getName());
@@ -116,74 +116,75 @@ public class ObjRelationshipInfo extends CayenneController implements
         // and target entities present, with DbEntities chosen.
         validateCanMap();
 
-        this.targetCollections = new ArrayList<String>(4);
+        this.targetCollections = new ArrayList<>(4);
         targetCollections.add(COLLECTION_TYPE_COLLECTION);
         targetCollections.add(ObjRelationship.DEFAULT_COLLECTION_TYPE);
         targetCollections.add(COLLECTION_TYPE_MAP);
         targetCollections.add(COLLECTION_TYPE_SET);
-        
-        for( String s : targetCollections) {
+
+        for (String s : targetCollections) {
             view.collectionTypeCombo.addItem(s);
         }
-       
-        this.mapKeys = new ArrayList<String>();
+
+        this.mapKeys = new ArrayList<>();
         initMapKeys();
 
         // setup path
-        dbRelationships = new ArrayList<DbRelationship>(relationship.getDbRelationships());
+        dbRelationships = new ArrayList<>(relationship.getDbRelationships());
         selectPath();
         updateCollectionChoosers();
-       
+
         // add dummy last relationship if we are not connected
         connectEnds();
         initFromModel();
         initController();
     }
-    
+
     private void initController() {
         view.getCancelButton().addActionListener(new ActionListener() {
-
+            @Override
             public void actionPerformed(ActionEvent e) {
                 view.dispose();
             }
         });
         view.getSaveButton().addActionListener(new ActionListener() {
-
+            @Override
             public void actionPerformed(ActionEvent e) {
-                saveMapping();
+                ObjRelationshipInfo.this.saveMapping();
             }
         });
         view.getNewRelButton().addActionListener(new ActionListener() {
-
+            @Override
             public void actionPerformed(ActionEvent e) {
-                createRelationship();
+                ObjRelationshipInfo.this.createRelationship();
             }
         });
         view.getSelectPathButton().addActionListener(new ActionListener() {
-
+            @Override
             public void actionPerformed(ActionEvent e) {
-                selectPath();
+                ObjRelationshipInfo.this.selectPath();
             }
-        });  
+        });
         view.getCollectionTypeCombo().addActionListener(new ActionListener() {
-            
+            @Override
             public void actionPerformed(ActionEvent e) {
-                setCollectionType();
+                ObjRelationshipInfo.this.setCollectionType();
             }
         });
         view.getMapKeysCombo().addActionListener(new ActionListener() {
-            
+            @Override
             public void actionPerformed(ActionEvent e) {
-                setMapKey();
+                ObjRelationshipInfo.this.setMapKey();
             }
         });
         view.getTargetCombo().addItemListener(new ItemListener() {
+            @Override
             public void itemStateChanged(ItemEvent e) {
                 if (e.getStateChange() == ItemEvent.SELECTED) {
                     Object targetName = e.getItem();
                     for (ObjEntity target : objectTargets) {
-                        if (target.getName() != null && target.getName().equals(targetName)) {
-                            setObjectTarget(target);
+                        if (Objects.equals(target.getName(), targetName)) {
+                            ObjRelationshipInfo.this.setObjectTarget(target);
                             return;
                         }
                     }
@@ -191,38 +192,19 @@ public class ObjRelationshipInfo extends CayenneController implements
             }
         });
     }
-    
+
     void initFromModel() {
 
         if (view.pathBrowser.getModel() == null) {
             EntityTreeModel treeModel = new EntityTreeModel(getStartEntity());
-            treeModel.setFilter(new EntityTreeFilter() {
-
-                public boolean attributeMatch(Object node, Attribute attr) {
-                    // attrs not allowed here
-                    return false;
-                }
-
-                public boolean relationshipMatch(Object node, Relationship rel) {
-                    if (!(node instanceof Relationship)) {
-                        return true;
-                    }
-
-                    /**
-                     * We do not allow A->B->A chains, where relationships are to-one
-                     */
-                    DbRelationship prev = (DbRelationship) node;
-                    return !(!rel.isToMany() && prev.getReverseRelationship() == rel);
-                }
-
-            });
+            treeModel.setFilter(new EntityTreeRelationshipFilter());
 
             view.pathBrowser.setModel(treeModel);
 
             setSelectionPath(getSavedDbRelationships());
         }
     }
-    
+
     /**
      * Selects path in browser
      */
@@ -234,30 +216,29 @@ public class ObjRelationshipInfo extends CayenneController implements
 
         view.pathBrowser.setSelectionPath(new TreePath(path));
     }
-    
+
     public void setCollectionType() {
-        setTargetCollection((String)view.collectionTypeCombo.getSelectedItem());
-        
-        if (COLLECTION_TYPE_MAP.equals(targetCollection)){
+        setTargetCollection((String) view.collectionTypeCombo.getSelectedItem());
+
+        if (COLLECTION_TYPE_MAP.equals(targetCollection)) {
             view.mapKeysLabel.setEnabled(true);
             view.mapKeysCombo.setEnabled(true);
             setMapKey();
-        }
-        else {
+        } else {
             view.mapKeysLabel.setEnabled(false);
             view.mapKeysCombo.setEnabled(false);
         }
     }
-    
+
     public void setMapKey() {
-        setMapKey((String)view.mapKeysCombo.getSelectedItem());
+        setMapKey((String) view.mapKeysCombo.getSelectedItem());
     }
 
     @Override
     public Component getView() {
         return view;
     }
- 
+
     public void setSavedDbRelationships(List<DbRelationship> rels) {
         this.savedDbRelationships = rels;
 
@@ -271,11 +252,11 @@ public class ObjRelationshipInfo extends CayenneController implements
         }
 
         currentPath = currPath;
-        view.currentPathLabel.setText(currPath);   
+        view.currentPathLabel.setText(currPath);
     }
-    
+
     public void selectPath() {
-        setSavedDbRelationships(new ArrayList<DbRelationship>(dbRelationships));
+        setSavedDbRelationships(new ArrayList<>(dbRelationships));
     }
 
     /**
@@ -285,7 +266,7 @@ public class ObjRelationshipInfo extends CayenneController implements
         setSelectionPath(getSavedDbRelationships());
         setDbRelationships(getSavedDbRelationships());
     }
-    
+
     /**
      * Updates 'collection type' and 'map keys' comboboxes
      */
@@ -298,15 +279,14 @@ public class ObjRelationshipInfo extends CayenneController implements
         }
 
         boolean mapKeysEnabled = collectionTypeEnabled
-                && ObjRelationshipInfo.COLLECTION_TYPE_MAP
-                        .equals(view.collectionTypeCombo.getSelectedItem());
+                && ObjRelationshipInfo.COLLECTION_TYPE_MAP.equals(view.collectionTypeCombo.getSelectedItem());
         view.mapKeysCombo.setEnabled(mapKeysEnabled);
         view.mapKeysLabel.setEnabled(mapKeysEnabled);
         if (mapKeysEnabled) {
             view.mapKeysCombo.setSelectedItem(mapKey);
         }
     }
-    
+
     /**
      * Clears paths and selections in browser
      */
@@ -317,86 +297,75 @@ public class ObjRelationshipInfo extends CayenneController implements
 
     protected void saveMapping() {
         if (!getDbRelationships().equals(getSavedDbRelationships())) {
-            if (JOptionPane.showConfirmDialog(
-                    (Component) getView(),
-                    "You have changed Db Relationship path. Do you want it to be saved?",
-                    "Save ObjRelationship",
+            if (JOptionPane.showConfirmDialog(getView(),
+                    "You have changed Db Relationship path. Do you want it to be saved?", "Save ObjRelationship",
                     JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
                 selectPath();
             }
         }
 
         if (savePath()) {
-            mediator.fireObjRelationshipEvent(new RelationshipEvent(Application
-                    .getFrame(), getRelationship(), getRelationship()
-                    .getSourceEntity()));
+            mediator.fireObjRelationshipEvent(new RelationshipEvent(Application.getFrame(), getRelationship(),
+                    getRelationship().getSourceEntity()));
         }
-        view.sourceEntityLabel.setText(relationship.getSourceEntity().getName()); 
+        view.sourceEntityLabel.setText(relationship.getSourceEntity().getName());
         view.dispose();
     }
-    
+
     /**
      * @return relationship path browser
      */
     public MultiColumnBrowser getPathBrowser() {
         return view.pathBrowser;
     }
-    
+
     /**
-     * Creates a new relationship connecting currently selected source entity with
-     * ObjRelationship target entity. User is allowed to edit the relationship, change its
-     * name, and create joins.
+     * Creates a new relationship connecting currently selected source entity
+     * with ObjRelationship target entity. User is allowed to edit the
+     * relationship, change its name, and create joins.
      */
     protected void createRelationship() {
-      
-        DbRelationship dbRel = getLastRelationship();
-        DbEntity source = dbRel != null ? (DbEntity) dbRel.getTargetEntity() : null;
 
-        DbRelationshipTarget targetModel = new DbRelationshipTarget(mediator,
-                getStartEntity(),
-                source);
+        DbRelationship dbRel = getLastRelationship();
+        DbEntity source = dbRel != null ? dbRel.getTargetEntity() : null;
+
+        DbRelationshipTarget targetModel = new DbRelationshipTarget(mediator, getStartEntity(), source);
         targetModel.startupAction();
 
         if (!targetModel.isSavePressed()) {
             return;
         }
 
-        DbRelationship dbRelationship = (DbRelationship) NamedObjectFactory
-                .createRelationship(
-                        targetModel.getSource(),
-                        targetModel.getTarget(),
-                        targetModel.isToMany());
-
-        // note: NamedObjectFactory doesn't set source or target, just the name
+        DbRelationship dbRelationship = new DbRelationship();
         dbRelationship.setSourceEntity(targetModel.getSource());
-        dbRelationship.setTargetEntity(targetModel.getTarget());
+        dbRelationship.setTargetEntityName(targetModel.getTarget());
         dbRelationship.setToMany(targetModel.isToMany());
+
+        dbRelationship.setName(createNamingStrategy(NameGeneratorPreferences
+                .getInstance()
+                .getLastUsedStrategies()
+                .get(0)).relationshipName(dbRelationship));
+
         targetModel.getSource().addRelationship(dbRelationship);
 
-        // TODO: creating relationship outside of ResolveDbRelationshipDialog confuses it
-        // to send incorrect event - CHANGE instead of ADD
-        ResolveDbRelationshipDialog dialog = new ResolveDbRelationshipDialog(
-                dbRelationship);
+        // TODO: creating relationship outside of ResolveDbRelationshipDialog
+        // confuses it to send incorrect event - CHANGE instead of ADD
+        ResolveDbRelationshipDialog dialog = new ResolveDbRelationshipDialog(dbRelationship);
 
         dialog.setVisible(true);
         if (dialog.isCancelPressed()) {
             targetModel.getSource().removeRelationship(dbRelationship.getName());
-        }
-        else {
+        } else {
             MultiColumnBrowser pathBrowser = getPathBrowser();
-            Object[] oldPath = targetModel.isSource1Selected() ? new Object[] {
-                    getStartEntity()
-                } : pathBrowser.getSelectionPath().getPath();
+            Object[] oldPath = targetModel.isSource1Selected()
+                    ? new Object[] { getStartEntity() }
+                    : pathBrowser.getSelectionPath().getPath();
 
-            /**
-             * Update the view
-             */
+            // Update the view
             EntityTreeModel treeModel = (EntityTreeModel) pathBrowser.getModel();
             treeModel.invalidate();
 
-            pathBrowser.setSelectionPath(new TreePath(new Object[] {
-                getStartEntity()
-            }));
+            pathBrowser.setSelectionPath(new TreePath(new Object[] { getStartEntity() }));
             pathBrowser.repaint();
 
             Object[] path = new Object[oldPath.length + 1];
@@ -408,7 +377,21 @@ public class ObjRelationshipInfo extends CayenneController implements
 
         dialog.dispose();
     }
-    
+
+    public ObjectNameGenerator createNamingStrategy(String strategyClass) {
+        try {
+            ClassLoadingService classLoader = application.getClassLoadingService();
+            return classLoader.loadClass(ObjectNameGenerator.class, strategyClass).newInstance();
+        } catch (Throwable th) {
+            JOptionPane.showMessageDialog(
+                    view,
+                    "Naming Strategy Initialization Error: " + th.getMessage(),
+                    "Naming Strategy Initialization Error",
+                    JOptionPane.ERROR_MESSAGE);
+            return null;
+        }
+    }
+
     /**
      * Sets list of DB Relationships current ObjRelationship is mapped to
      */
@@ -421,15 +404,7 @@ public class ObjRelationshipInfo extends CayenneController implements
             return;
         }
 
-        Relationship rel = (Relationship) selectedPath.getLastPathComponent();
-        DbEntity target = (DbEntity) rel.getTargetEntity();
-         /**
-         * Initialize root with one of mapped ObjEntities.
-         */
-        Collection<ObjEntity> objEntities = target.getDataMap().getMappedEntities(target);
-
-        List<DbRelationship> relPath = new Vector<DbRelationship>(selectedPath
-                .getPathCount() - 1);
+        List<DbRelationship> relPath = new ArrayList<>(selectedPath.getPathCount() - 1);
         for (int i = 1; i < selectedPath.getPathCount(); i++) {
             relPath.add((DbRelationship) selectedPath.getPathComponent(i));
         }
@@ -437,22 +412,22 @@ public class ObjRelationshipInfo extends CayenneController implements
 
         updateCollectionChoosers();
     }
+
     public void setObjectTarget(ObjEntity objectTarget) {
         if (this.objectTarget != objectTarget) {
-             this.objectTarget = objectTarget;
+            this.objectTarget = objectTarget;
 
             // init available map keys
             initMapKeys();
         }
     }
+
     private void initMapKeys() {
         this.mapKeys.clear();
 
         mapKeys.add(DEFAULT_MAP_KEY);
 
-        /**
-         * Object target can be null when selected target DbEntity has no ObjEntities
-         */
+        // Object target can be null when selected target DbEntity has no ObjEntities
         if (objectTarget == null) {
             return;
         }
@@ -461,15 +436,15 @@ public class ObjRelationshipInfo extends CayenneController implements
             mapKeys.add(attribute.getName());
         }
         view.mapKeysCombo.removeAllItems();
-        for(String s :mapKeys)
+        for (String s : mapKeys)
             view.mapKeysCombo.addItem(s);
-        
+
         if (mapKey != null && !mapKeys.contains(mapKey)) {
             mapKey = DEFAULT_MAP_KEY;
             view.mapKeysCombo.setSelectedItem(mapKey);
         }
     }
-    
+
     /**
      * Places in objectTargets list all ObjEntities for specified DbEntity
      */
@@ -477,15 +452,15 @@ public class ObjRelationshipInfo extends CayenneController implements
     protected void updateTargetCombo(DbEntity dbTarget) {
         // copy those that have DbEntities mapped to dbTarget, and then sort
 
-        this.objectTargets = new ArrayList<ObjEntity>();
+        this.objectTargets = new ArrayList<>();
 
         if (dbTarget != null) {
             objectTargets.addAll(dbTarget.getDataMap().getMappedEntities(dbTarget));
             Collections.sort(objectTargets, Comparators.getNamedObjectComparator());
         }
         view.targetCombo.removeAllItems();
-        for( ObjEntity s : objectTargets) {
-           view.targetCombo.addItem(s.getName());
+        for (ObjEntity s : objectTargets) {
+            view.targetCombo.addItem(s.getName());
         }
     }
 
@@ -508,11 +483,11 @@ public class ObjRelationshipInfo extends CayenneController implements
     }
 
     /**
-     * @return last relationship in the path, or <code>null</code> if path is empty
+     * @return last relationship in the path, or <code>null</code> if path is
+     *         empty
      */
     public DbRelationship getLastRelationship() {
-        return dbRelationships.size() == 0 ? null : dbRelationships.get(dbRelationships
-                .size() - 1);
+        return dbRelationships.size() == 0 ? null : dbRelationships.get(dbRelationships.size() - 1);
     }
 
     /**
@@ -521,13 +496,10 @@ public class ObjRelationshipInfo extends CayenneController implements
     public void setDbRelationships(List<DbRelationship> rels) {
         this.dbRelationships = rels;
 
-        updateTargetCombo(rels.size() > 0 ? (DbEntity) rels
-                .get(rels.size() - 1)
-                .getTargetEntity() : null);
-        
+        updateTargetCombo(rels.size() > 0 ? rels.get(rels.size() - 1).getTargetEntity() : null);
         updateCollectionChoosers();
     }
-    
+
     /**
      * Returns currently selected target of the ObjRelationship.
      */
@@ -553,7 +525,7 @@ public class ObjRelationshipInfo extends CayenneController implements
     /**
      * Processes relationship path when path component at index was changed.
      */
-    public synchronized void relationshipChanged(int index) {
+    public void relationshipChanged(int index) {
         // strip everything starting from the index
         breakChain(index);
 
@@ -562,7 +534,8 @@ public class ObjRelationshipInfo extends CayenneController implements
     }
 
     public boolean isToMany() {
-        // copied algorithm from ObjRelationship.calculateToMany(), only iterating through
+        // copied algorithm from ObjRelationship.calculateToMany(), only
+        // iterating through
         // the unsaved dbrels selection.
 
         for (DbRelationship relationship : dbRelationships) {
@@ -573,14 +546,15 @@ public class ObjRelationshipInfo extends CayenneController implements
 
         return false;
     }
-    
+
     /**
      * Stores current state of the model in the internal ObjRelationship.
      */
-    public synchronized boolean savePath() {
+    public boolean savePath() {
         boolean hasChanges = false;
 
         boolean oldToMany = relationship.isToMany();
+        boolean oldPathNotEmpty = !relationship.getDbRelationships().isEmpty();
 
         String relationshipName = getRelationshipName();
         if (!Util.nullSafeEquals(relationship.getName(), relationshipName)) {
@@ -589,33 +563,26 @@ public class ObjRelationshipInfo extends CayenneController implements
         }
 
         if (savedDbRelationships.size() > 0) {
-            DbEntity lastEntity = (DbEntity) savedDbRelationships.get(
-                    savedDbRelationships.size() - 1).getTargetEntity();
+            DbEntity lastEntity = savedDbRelationships.get(savedDbRelationships.size() - 1).getTargetEntity();
 
             if (objectTarget == null || objectTarget.getDbEntity() != lastEntity) {
-                /**
-                 * Entities in combobox and path browser do not match. In this case, we
-                 * rely on the browser and automatically select one of lastEntity's
-                 * ObjEntities
+                /*
+                 * Entities in combobox and path browser do not match. In this
+                 * case, we rely on the browser and automatically select one of
+                 * lastEntity's ObjEntities
                  */
-                Collection<ObjEntity> objEntities = lastEntity
-                        .getDataMap()
-                        .getMappedEntities(lastEntity);
-                objectTarget = objEntities.size() == 0 ? null : objEntities
-                        .iterator()
-                        .next();
+                Collection<ObjEntity> objEntities = lastEntity.getDataMap().getMappedEntities(lastEntity);
+                objectTarget = objEntities.size() == 0 ? null : objEntities.iterator().next();
             }
         }
 
-        if (objectTarget == null
-                || !Util.nullSafeEquals(objectTarget.getName(), relationship
-                        .getTargetEntityName())) {
+        if (objectTarget == null || !Util.nullSafeEquals(objectTarget.getName(), relationship.getTargetEntityName())) {
             hasChanges = true;
 
             // note on events notification - this needs to be propagated
             // via old modeler events, but we leave this to the controller
             // since model knows nothing about Modeler mediator.
-            relationship.setTargetEntity(objectTarget);
+            relationship.setTargetEntityName(objectTarget);
         }
 
         // check for path modifications
@@ -623,8 +590,7 @@ public class ObjRelationshipInfo extends CayenneController implements
         if (oldPath.size() != savedDbRelationships.size()) {
             hasChanges = true;
             updatePath();
-        }
-        else {
+        } else {
             for (int i = 0; i < oldPath.size(); i++) {
                 DbRelationship next = savedDbRelationships.get(i);
 
@@ -636,8 +602,7 @@ public class ObjRelationshipInfo extends CayenneController implements
             }
         }
 
-        String collectionType = ObjRelationship.DEFAULT_COLLECTION_TYPE
-                .equals(targetCollection)
+        String collectionType = ObjRelationship.DEFAULT_COLLECTION_TYPE.equals(targetCollection)
                 || !relationship.isToMany() ? null : targetCollection;
         if (!Util.nullSafeEquals(collectionType, relationship.getCollectionType())) {
             hasChanges = true;
@@ -645,18 +610,18 @@ public class ObjRelationshipInfo extends CayenneController implements
         }
 
         // map key only makes sense for Map relationships
-        String mapKey = COLLECTION_TYPE_MAP.equals(collectionType)
-                && !DEFAULT_MAP_KEY.equals(this.mapKey) ? this.mapKey : null;
+        String mapKey = COLLECTION_TYPE_MAP.equals(collectionType) && !DEFAULT_MAP_KEY.equals(this.mapKey) ? this.mapKey
+                : null;
         if (!Util.nullSafeEquals(mapKey, relationship.getMapKey())) {
             hasChanges = true;
             relationship.setMapKey(mapKey);
         }
 
-        /**
-         * As of CAY-436 here we check if to-many property has changed during the editing,
-         * and if so, delete rule must be reset to default value
+        /*
+         * As of CAY-436 here we check if to-many property has changed during
+         * the editing, and if so, delete rule must be reset to default value
          */
-        if (hasChanges && relationship.isToMany() != oldToMany) {
+        if (oldPathNotEmpty && hasChanges && relationship.isToMany() != oldToMany) {
             DeleteRuleUpdater.updateObjRelationship(relationship);
         }
 
@@ -677,12 +642,12 @@ public class ObjRelationshipInfo extends CayenneController implements
 
     private void breakChain(int index) {
         // strip everything starting from the index
-
         while (dbRelationships.size() > (index + 1)) {
             // remove last
             dbRelationships.remove(dbRelationships.size() - 1);
         }
     }
+
     // Connects last selected DbRelationship in the path to the
     // last DbEntity, creating a dummy relationship if needed.
     private void connectEnds() {
@@ -697,13 +662,9 @@ public class ObjRelationshipInfo extends CayenneController implements
 
         if (target != null && (last == null || last.getTargetEntity() != target)) {
             // try to connect automatically, if we can't use dummy connector
-
             Entity source = (last == null) ? getStartEntity() : last.getTargetEntity();
             if (source != null) {
-
-                Relationship anyConnector = source != null ? source
-                        .getAnyRelationship(target) : null;
-
+                Relationship anyConnector = source.getAnyRelationship(target);
                 if (anyConnector != null) {
                     dbRelationships.add((DbRelationship) anyConnector);
                 }
@@ -712,28 +673,29 @@ public class ObjRelationshipInfo extends CayenneController implements
     }
 
     /**
-     * Checks if the entity can be edited with this inspector. NOTE: As of CAY-1077,
-     * relationship inspector can be opened even if no target entity was set.
+     * Checks if the entity can be edited with this inspector. NOTE: As of
+     * CAY-1077, relationship inspector can be opened even if no target entity
+     * was set.
      */
     private void validateCanMap() {
         if (relationship.getSourceEntity() == null) {
-            throw new CayenneRuntimeException(
-                    "Can't map relationship without source entity.");
+            throw new CayenneRuntimeException("Can't map relationship without source entity.");
         }
 
         if (getStartEntity() == null) {
-            throw new CayenneRuntimeException(
-                    "Can't map relationship without source DbEntity.");
+            JOptionPane.showMessageDialog(getView(), "Can't map relationship without source DbEntity. Set source DbEntity.");
+            throw new CayenneRuntimeException("Can't map relationship without source DbEntity.");
         }
     }
 
     public DbEntity getStartEntity() {
-        return ((ObjEntity) relationship.getSourceEntity()).getDbEntity();
+        return relationship.getSourceEntity().getDbEntity();
     }
 
     public DbEntity getEndEntity() {
-        /**
-         * Object target can be null when selected target DbEntity has no ObjEntities
+        /*
+         * Object target can be null when selected target DbEntity has no
+         * ObjEntities
          */
         if (objectTarget == null) {
             return null;
