@@ -19,27 +19,17 @@
 
 package org.apache.cayenne.modeler.editor;
 
-import java.awt.BorderLayout;
-import java.awt.event.ActionEvent;
-import java.awt.event.ActionListener;
-import java.util.Arrays;
-
-import javax.swing.DefaultComboBoxModel;
-import javax.swing.JButton;
-import javax.swing.JCheckBox;
-import javax.swing.JComboBox;
-import javax.swing.JLabel;
-import javax.swing.JPanel;
-import javax.swing.JTextField;
-
+import com.jgoodies.forms.builder.DefaultFormBuilder;
+import com.jgoodies.forms.layout.FormLayout;
 import org.apache.cayenne.configuration.DataChannelDescriptor;
 import org.apache.cayenne.configuration.DataNodeDescriptor;
 import org.apache.cayenne.configuration.event.DataMapEvent;
-import org.apache.cayenne.configuration.event.DataNodeEvent;
 import org.apache.cayenne.map.DataMap;
 import org.apache.cayenne.map.ObjEntity;
 import org.apache.cayenne.modeler.Application;
 import org.apache.cayenne.modeler.ProjectController;
+import org.apache.cayenne.modeler.action.LinkDataMapAction;
+import org.apache.cayenne.modeler.dialog.datamap.CatalogUpdateController;
 import org.apache.cayenne.modeler.dialog.datamap.LockingUpdateController;
 import org.apache.cayenne.modeler.dialog.datamap.PackageUpdateController;
 import org.apache.cayenne.modeler.dialog.datamap.SchemaUpdateController;
@@ -54,8 +44,17 @@ import org.apache.cayenne.modeler.util.TextAdapter;
 import org.apache.cayenne.util.Util;
 import org.apache.cayenne.validation.ValidationException;
 
-import com.jgoodies.forms.builder.DefaultFormBuilder;
-import com.jgoodies.forms.layout.FormLayout;
+import javax.swing.DefaultComboBoxModel;
+import javax.swing.JButton;
+import javax.swing.JCheckBox;
+import javax.swing.JComboBox;
+import javax.swing.JLabel;
+import javax.swing.JPanel;
+import javax.swing.JTextField;
+import java.awt.BorderLayout;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
+import java.util.Arrays;
 
 /**
  * Panel for editing a DataMap.
@@ -68,11 +67,13 @@ public class DataMapView extends JPanel {
     protected JLabel location;
     protected JComboBox nodeSelector;
     protected JCheckBox defaultLockType;
+    protected TextAdapter defaultCatalog;
     protected TextAdapter defaultSchema;
     protected TextAdapter defaultPackage;
     protected TextAdapter defaultSuperclass;
     protected JCheckBox quoteSQLIdentifiers;
 
+    protected JButton updateDefaultCatalog;
     protected JButton updateDefaultSchema;
     protected JButton updateDefaultPackage;
     protected JButton updateDefaultSuperclass;
@@ -82,6 +83,7 @@ public class DataMapView extends JPanel {
     protected JCheckBox clientSupport;
 
     protected JLabel defaultClientPackageLabel;
+    protected JLabel defaultClientSuperclassLabel;
     protected TextAdapter defaultClientPackage;
     protected TextAdapter defaultClientSuperclass;
     protected JButton updateDefaultClientPackage;
@@ -107,6 +109,14 @@ public class DataMapView extends JPanel {
         nodeSelector = Application.getWidgetFactory().createUndoableComboBox();
         nodeSelector.setRenderer(CellRenderers.listRendererWithIcons());
 
+        updateDefaultCatalog = new JButton("Update...");
+        defaultCatalog = new TextAdapter(new JTextField()) {
+
+            protected void updateModel(String text) {
+                setDefaultCatalog(text);
+            }
+        };
+        
         updateDefaultSchema = new JButton("Update...");
         defaultSchema = new TextAdapter(new JTextField()) {
 
@@ -167,6 +177,7 @@ public class DataMapView extends JPanel {
         builder.append("Quote SQL Identifiers:", quoteSQLIdentifiers, 3);
 
         builder.appendSeparator("Entity Defaults");
+        builder.append("DB Catalog:", defaultCatalog.getComponent(), updateDefaultCatalog);
         builder.append("DB Schema:", defaultSchema.getComponent(), updateDefaultSchema);
         builder.append(
                 "Java Package:",
@@ -184,7 +195,7 @@ public class DataMapView extends JPanel {
                 "Client Java Package:",
                 defaultClientPackage.getComponent(),
                 updateDefaultClientPackage);
-        builder.append(
+        defaultClientSuperclassLabel = builder.append(
                 "Custom Superclass:",
                 defaultClientSuperclass.getComponent(),
                 updateDefaultClientSuperclass);
@@ -247,6 +258,13 @@ public class DataMapView extends JPanel {
                 updateDefaultClientSuperclass();
             }
         });
+        
+        updateDefaultCatalog.addActionListener(new ActionListener() {
+
+            public void actionPerformed(ActionEvent e) {
+                updateDefaultCatalog();
+            }
+        });
 
         updateDefaultSchema.addActionListener(new ActionListener() {
 
@@ -319,6 +337,7 @@ public class DataMapView extends JPanel {
         // init default fields
         defaultLockType.setSelected(map.getDefaultLockType() != ObjEntity.LOCK_TYPE_NONE);
         defaultPackage.setText(map.getDefaultPackage());
+        defaultCatalog.setText(map.getDefaultCatalog());
         defaultSchema.setText(map.getDefaultSchema());
         defaultSuperclass.setText(map.getDefaultSuperclass());
 
@@ -334,6 +353,7 @@ public class DataMapView extends JPanel {
         updateDefaultClientPackage.setEnabled(enabled);
         defaultClientPackageLabel.setEnabled(enabled);
 
+        defaultClientSuperclassLabel.setEnabled(enabled);
         defaultClientSuperclass.getComponent().setEnabled(enabled);
         updateDefaultClientSuperclass.setEnabled(enabled);
     }
@@ -455,6 +475,26 @@ public class DataMapView extends JPanel {
         dataMap.setDefaultClientSuperclass(newSuperclass);
         eventController.fireDataMapEvent(new DataMapEvent(this, dataMap));
     }
+    
+    void setDefaultCatalog(String newCatalog) {
+        DataMap dataMap = eventController.getCurrentDataMap();
+
+        if (dataMap == null) {
+            return;
+        }
+
+        if (newCatalog != null && newCatalog.trim().length() == 0) {
+            newCatalog = null;
+        }
+
+        String oldCatalog = dataMap.getDefaultCatalog();
+        if (Util.nullSafeEquals(newCatalog, oldCatalog)) {
+            return;
+        }
+
+        dataMap.setDefaultCatalog(newCatalog);
+        eventController.fireDataMapEvent(new DataMapEvent(this, dataMap));
+    }
 
     void setDefaultSchema(String newSchema) {
         DataMap dataMap = eventController.getCurrentDataMap();
@@ -536,34 +576,19 @@ public class DataMapView extends JPanel {
     void setDataNode() {
         DataNodeDescriptor node = (DataNodeDescriptor) nodeSelector.getSelectedItem();
         DataMap map = eventController.getCurrentDataMap();
+        LinkDataMapAction action = eventController.getApplication().getActionManager().getAction(LinkDataMapAction.class);
+        action.linkDataMap(map, node);
+    }
+    
+    void updateDefaultCatalog() {
+        DataMap dataMap = eventController.getCurrentDataMap();
 
-        // no change?
-        if (node != null && node.getDataMapNames().contains(map.getName())) {
+        if (dataMap == null) {
             return;
         }
 
-        // unlink map from any nodes
-
-        for (DataNodeDescriptor nextNode : ((DataChannelDescriptor) eventController
-                .getProject()
-                .getRootNode()).getNodeDescriptors()) {
-
-            // Theoretically only one node may contain a datamap at each given time.
-            // Being paranoid, we will still scan through all.
-            if (nextNode != node && nextNode.getDataMapNames().contains(map.getName())) {
-                nextNode.getDataMapNames().remove(map.getName());
-
-                // announce DataNode change
-                eventController.fireDataNodeEvent(new DataNodeEvent(this, nextNode));
-            }
-        }
-
-        // link to a selected node
-        if (node != null) {
-            node.getDataMapNames().add(map.getName());
-
-            // announce DataNode change
-            eventController.fireDataNodeEvent(new DataNodeEvent(this, node));
+        if (dataMap.getDbEntities().size() > 0 || dataMap.getProcedures().size() > 0) {
+            new CatalogUpdateController(eventController, dataMap).startupAction();
         }
     }
 
