@@ -29,6 +29,7 @@ import java.util.EventObject;
 import java.util.LinkedList;
 import java.util.List;
 
+import javax.swing.BorderFactory;
 import javax.swing.DefaultComboBoxModel;
 import javax.swing.JButton;
 import javax.swing.JCheckBox;
@@ -53,6 +54,7 @@ import org.apache.cayenne.modeler.ProjectController;
 import org.apache.cayenne.modeler.action.ActionManager;
 import org.apache.cayenne.modeler.action.CreateAttributeAction;
 import org.apache.cayenne.modeler.action.CreateRelationshipAction;
+import org.apache.cayenne.modeler.action.ObjEntityCounterpartAction;
 import org.apache.cayenne.modeler.action.ObjEntitySyncAction;
 import org.apache.cayenne.modeler.dialog.objentity.ClassNameUpdater;
 import org.apache.cayenne.modeler.dialog.validator.DuplicatedAttributesDialog;
@@ -62,7 +64,6 @@ import org.apache.cayenne.modeler.graph.action.ShowGraphEntityAction;
 import org.apache.cayenne.modeler.util.CellRenderers;
 import org.apache.cayenne.modeler.util.Comparators;
 import org.apache.cayenne.modeler.util.ExpressionConvertor;
-import org.apache.cayenne.modeler.util.ModelerUtil;
 import org.apache.cayenne.modeler.util.TextAdapter;
 import org.apache.cayenne.modeler.util.combo.AutoCompletion;
 import org.apache.cayenne.util.CayenneMapEntry;
@@ -102,7 +103,6 @@ public class ObjEntityTab extends JPanel implements ObjEntityDisplayListener,
     protected TextAdapter superClassName;
     protected TextAdapter qualifier;
     protected JComboBox dbEntityCombo;
-    protected JButton syncWithDbEntityButton;
     protected JComboBox superEntityCombo;
     protected JButton tableLabel;
     protected JCheckBox readOnly;
@@ -131,16 +131,18 @@ public class ObjEntityTab extends JPanel implements ObjEntityDisplayListener,
         this.setLayout(new BorderLayout());
 
         JToolBar toolBar = new JToolBar();
+        toolBar.setBorder(BorderFactory.createEmptyBorder());
+        toolBar.setFloatable(false);
         ActionManager actionManager = Application.getInstance().getActionManager();
 
-        toolBar.add(actionManager.getAction(ObjEntitySyncAction.class).buildButton());
-        toolBar.add(actionManager.getAction(CreateAttributeAction.class).buildButton());
-        toolBar
-                .add(actionManager
-                        .getAction(CreateRelationshipAction.class)
-                        .buildButton());
+        toolBar.add(actionManager.getAction(CreateAttributeAction.class).buildButton(1));
+        toolBar.add(actionManager.getAction(CreateRelationshipAction.class).buildButton(3));
+        toolBar.addSeparator();
+        toolBar.add(actionManager.getAction(ObjEntitySyncAction.class).buildButton(1));
+        toolBar.add(actionManager.getAction(ObjEntityCounterpartAction.class).buildButton(3));
         toolBar.addSeparator();
         toolBar.add(actionManager.getAction(ShowGraphEntityAction.class).buildButton());
+
         add(toolBar, BorderLayout.NORTH);
 
         // create widgets
@@ -192,9 +194,6 @@ public class ObjEntityTab extends JPanel implements ObjEntityDisplayListener,
         tableLabel.setMargin(new Insets(0, 0, 0, 0));
         tableLabel.setBorder(null);
 
-        syncWithDbEntityButton = new JButton("Sync w/DbEntity");
-        syncWithDbEntityButton.setIcon(ModelerUtil.buildIcon("icon-sync.gif"));
-        syncWithDbEntityButton.setToolTipText("Sync this ObjEntity with its DBEntity");
 
         isAbstract = new JCheckBox();
         serverOnly = new JCheckBox();
@@ -214,37 +213,31 @@ public class ObjEntityTab extends JPanel implements ObjEntityDisplayListener,
         };
 
         // assemble
-        FormLayout layout = new FormLayout(
-                "right:100dlu, 3dlu, fill:135dlu, 3dlu, pref",
-                "");
+        FormLayout layout = new FormLayout("right:pref, 3dlu, fill:200dlu", "");
         DefaultFormBuilder builder = new DefaultFormBuilder(layout);
         builder.setDefaultDialogBorder();
 
         builder.appendSeparator("ObjEntity Configuration");
-        builder.append("ObjEntity Name:", name.getComponent(), 3);
-        builder.append("Inheritance:", superEntityCombo, 3);
-        builder.append(tableLabel, dbEntityCombo, syncWithDbEntityButton);
-        isAbstractLabel = builder.append("Abstract class:", isAbstract, 3);
-
+        builder.append("ObjEntity Name:", name.getComponent());
+        builder.append("Inheritance:", superEntityCombo);
+        builder.append(tableLabel, dbEntityCombo);
+        isAbstractLabel = builder.append("Abstract class:", isAbstract);
         builder.appendSeparator();
 
-        builder.append("Java Class:", className.getComponent(), 3);
-        superclassLabel = builder.append("Superclass:", superClassName.getComponent(), 3);
-        builder.append("Qualifier:", qualifier.getComponent(), 3);
-        builder.append("Read-Only:", readOnly, 3);
-        builder.append("Optimistic Locking:", optimisticLocking, 3);
+        builder.append("Java Class:", className.getComponent());
+     
+        superclassLabel = builder.append("Superclass:", superClassName.getComponent());
+        builder.append("Qualifier:", qualifier.getComponent());
+        builder.append("Read-Only:", readOnly);
+        builder.append("Optimistic Locking:", optimisticLocking);
         // add callback-related stuff
-        builder.append("Exclude superclass listeners:", excludeSuperclassListeners, 3);
-        builder.append("Exclude default listeners:", excludeDefaultListeners, 3);
+        builder.append("Exclude superclass listeners:", excludeSuperclassListeners);
+        builder.append("Exclude default listeners:", excludeDefaultListeners);
 
         clientSeparator = builder.appendSeparator("Java Client");
-        serverOnlyLabel = builder.append("Not for Client Use:", serverOnly, 3);
-        clientClassNameLabel = builder.append("Client Java Class:", clientClassName
-                .getComponent(), 3);
-        clientSuperClassNameLabel = builder.append(
-                "Client Superclass:",
-                clientSuperClassName.getComponent(),
-                3);
+        serverOnlyLabel = builder.append("Not for Client Use:", serverOnly);
+        clientClassNameLabel = builder.append("Client Java Class:", clientClassName.getComponent());
+        clientSuperClassNameLabel = builder.append("Client Superclass:", clientSuperClassName.getComponent());
 
         add(builder.getPanel(), BorderLayout.CENTER);
     }
@@ -260,7 +253,7 @@ public class ObjEntityTab extends JPanel implements ObjEntityDisplayListener,
                 // Change DbEntity for current ObjEntity
                 ObjEntity entity = mediator.getCurrentObjEntity();
                 DbEntity dbEntity = (DbEntity) dbEntityCombo.getSelectedItem();
-                syncWithDbEntityButton.setEnabled(dbEntity != null);
+       
 
                 if (dbEntity != entity.getDbEntity()) {
                     entity.setDbEntity(dbEntity);
@@ -302,6 +295,15 @@ public class ObjEntityTab extends JPanel implements ObjEntityDisplayListener,
 
                     }
                     entity.setSuperEntityName(name);
+
+                    // drop not valid dbAttributePath
+                    if (name == null) {
+                        for (ObjAttribute objAttribute : entity.getAttributes()) {
+                            if (objAttribute.getDbAttribute() == null) {
+                                objAttribute.setDbAttributePath(null);
+                            }
+                        }
+                    }
                     
                     if (name == null) {
                         dbEntityCombo.setEnabled(true);
@@ -353,8 +355,7 @@ public class ObjEntityTab extends JPanel implements ObjEntityDisplayListener,
             }
         });
 
-        syncWithDbEntityButton.addActionListener(new ObjEntitySyncAction(mediator
-                .getApplication()));
+ 
 
         readOnly.addActionListener(new ActionListener() {
 
@@ -474,7 +475,6 @@ public class ObjEntityTab extends JPanel implements ObjEntityDisplayListener,
         dbEntityCombo.setEnabled(!isUsedInheritance);
 
         // toggle visibilty and editability rules
-        
         toggleClientFieldsVisible(map.isClientSupported());
         toggleEnabled(entity.getSuperEntityName() == null, !entity.isServerOnly());
 
@@ -649,8 +649,6 @@ public class ObjEntityTab extends JPanel implements ObjEntityDisplayListener,
         superClassName.getComponent().setEnabled(directTableMapping);
         superclassLabel.setEnabled(directTableMapping);
 
-        syncWithDbEntityButton.setEnabled(dbEntityCombo.getSelectedItem() != null);
-
         clientSuperClassName.getComponent().setEnabled(
                 directTableMapping && clientFieldsEnabled);
         clientSuperClassNameLabel.setEnabled(directTableMapping && clientFieldsEnabled);
